%Descripcion:
%	Controla en uicontrol edit y slider de el interfaz principal 
function varargout=CBEditor(orden,varargin)
%variables persistentes
%	buffer: lista de lineas del fichero
%	primera: primera lnea mostrada
%	ultima: ltima linea mostrada
%  numLineas: nmero total de lineas=length(buffer)
%  visualizadas: nmero de lneas que se pueden mostrar a la vez
%	hf: handler a la figure del InterfazPrincipal
%	he: handler al uicontrol edit
%	hs: handler al uicontrol slider
persistent buffer primera ultima numLineas visualizadas hf he hs

%actuar segn el tipo de orden
switch orden
case 'start'
   %buscar he y hs
   hf=findobj('tag','IP');
   he=findobj('tag','IPTexto');
   hs=findobj('tag','IPSlider');
   
   %inicializar las dems variables
   buffer={};
   linea=0;
   ultima=0;
   numLineas=0;
   
   %calcular el nmero de lneas que se pueden visualizar
   posicion=get(he,'position');
   fuente=12.1;	%valor empirico en lugar de get(he,'fontsize');
   visualizadas=floor(posicion(4)/fuente);
   
   %volver
   return
   
case 'open'
   %poner error por defecto
   varargout{1}='';
   
   %inicializar las variables
   buffer={};
   linea=0;
   ultima=0;
   numLineas=0;
   
   %leer los datos
   ruta=varargin{1};
   fichero=varargin{2};
   
   %abre un fichero y los muestra
	fid=fopen([ruta fichero],'r');
   if fid==-1
      varargout{1}=['no se puede abrir el fichero ' fichero];
      return
   end

	%poner el nombre del fichero en el titulo de la ventana
	set(hf,'name',fichero);

	%leer su contenido
	while 1
   	%leer linea
   	linea=fgets(fid);
      
   	%comprobar si se ha llegado al final del fichero
   	if ~ischar(linea),break,end   
      
      %incrementar el nmero de lneas
      numLineas=numLineas+1;
      
   	%aadir linea quitando \n (caracteres 13 y 10)
   	if length(linea)>=2 & linea(end-1)==char(13) & linea(end)==char(10)
         buffer{end+1}=linea(1:end-2);
      else
         buffer{end+1}=linea;
      end   
	end
      
   %cerrar el fichero
	fclose(fid);
   
   %mostrar lneas si el buffer no est vaco
   if numLineas>0
      %Poner el nmero de lnea
	  	cifras=floor(1+log10(numLineas));
      espacios=8-cifras; %esperemos que el documento no tenga ms de 1e8 lneas  
      
      ceros=char('0'*ones(1,cifras));
      huecos=char(' '*ones(1,espacios));
      for i=1:numLineas
         prefijo=[ceros num2str(i)];
         buffer{i}=[prefijo(end-cifras+1:end) huecos buffer{i}];
      end     
        
   	%dar valores a las variables que controlan como se muestra el fichero
  		primera=1;
   	if numLineas>visualizadas
      	%hay ms lineas de las que se pueden mostrar
         ultima=visualizadas;
         
         %determinar los pasos del slider
         %cuando se pulse la flecha se mueve una lnea
         %cuando se pulse el knob se mueve diez lneas
         %el valor del slider indica la primera linea a ver
         max=numLineas-visualizadas+1;
         min=1;
         step=[1/(max-min) 10/(max-min)];
         
         %poner los valores del slider
         set(hs,'max',max,'min',1,'value',max,'sliderstep',step,'enable','on');
      else
         %el nmero de lneas es inferior al que se pueden mostrar
      	ultima=numLineas;
      	set(hs,'enable','off','max',1,'min',0,'value',1);
   	end
      
		%escribir su contenido en el uicontrol edit
		set(he,'string',{buffer{primera:ultima}});
		%set(he,'string',buffer);

		%volver
      return
   else
      %fichero vacio
      set(hs,'max',1,'min',0,'value',1,'enable','off');
      set(he,'string',buffer);
      
      %volver
      return
   end
   
case 'click'
   %siempre que se llama a click podemos asegurar que
   %	1) el slider est activado
   %	2) numLineas>visualizadas
   %se ha hecho click en el slider. Ajusta el tamao a mostrar
   valor=floor(get(hs,'value'));
   
   %ajustar la primera y la ultima lnea que se ven
   %Esta formula es as pq cuando el knob est arriba toma el valor
   %maxmo y cuando est abajo toma el valor mnimo. La frmula 
   %que aplico para resolver cul es la primera lnea es:
   %	salida=max+min-entrada 
   %donde:
   %	salida es primera
   %	entrada es get(hs,'value')
   max=numLineas-visualizadas+1;
   min=1;
   primera=max+min-floor(get(hs,'value'));
   ultima=primera+visualizadas-1;
   
   %escribir su contenido en el uicontrol edit
	set(he,'string',{buffer{primera:ultima}});
   
   %volver
   return
end
