%CBIOCerrar Funcin de Callback del cuadro de dilogo InterfazOpciones.
%           Realiza las siguientes operaciones:
%               1) Cierra el cuadro de dilogo de InterfazOpciones
%               2) Comprueba que los valores asignados a las opciones son 
%                  correctos.
%               3) Salva las opciones en el fichero <moo>\gui\opcionesmoo.mat
%
%Sintaxis:
%       CBIOCerrar(salvar)
%Entrada:
%       salvar: variable boolean. 
%               Si toma el valor 1 realiza las operaciones de (1)-(3)
%               Si toma el valor 0 slo cierra el cuadro de dilogo

%REVISIONES
%   17/02/09    ABP
%               1. Hacer relativo el directorio <moo>
%               2. Sustituir | por || en los ifs
function CBIOCerrar(salvar)
if salvar
    %---leer opciones
    alias=get(findobj('tag','IOAlias'),'value');
	matrices=get(findobj('tag','IOMatrices'),'value');
	pantelides=get(findobj('tag','IOPantelides'),'string');
	noedo=get(findobj('tag','IONoEdo'),'string');
   
    dirtrabajo=get(findobj('tag','IODirTrabajo'),'string');
    dirlibreria=get(findobj('tag','IODirLibreria'),'string');
   
    metodointegracion=get(findobj('tag','IOMetodoIntegracion'),'value');
    reltol=get(findobj('tag','IORelTol'),'string');
    abstol=get(findobj('tag','IOAbsTol'),'string');
    maxstep=get(findobj('tag','IOMaxStep'),'string');
    initialstep=get(findobj('tag','IOInitialStep'),'string');
   
	%---comprobar que son correctos los valores
	pantelides=str2double(pantelides);
    if isnan(pantelides) || pantelides<=0 
        errordlg('Incorrect Pantelides value','ERROR!','modal');
        return
    end
   
    noedo=str2double(noedo);
    if isnan(noedo) || noedo<=0
        errordlg('Incorrect number of EDOs','ERROR!','modal');
        return
    end
      
    if metodointegracion<0 || metodointegracion>2
        errordlg('Incorrect integration method','ERROR!','modal');
        return
    end

    reltol=str2double(reltol);
    if isnan(reltol) || reltol<=0
        errordlg('Incorrect RelTol value','ERROR!','modal');
        return
    end
   
    abstol=str2double(abstol);
    if isnan(abstol) || abstol<=0
	  	errordlg('Incorrect AbsTol value','ERROR!','modal');
        return
    end
   
    if ~isempty(maxstep)
		maxstep=str2double(maxstep);
        if isnan(maxstep) || maxstep<=0
            errordlg('Incorrect MaxStep value','ERROR!','modal');
            return
        end
    end
   
    if ~isempty(initialstep)
        initialstep=str2double(initialstep);
        if isnan(initialstep) || initialstep<=0
	  		errordlg('Incorrect RelTol value','ERROR!','modal');
            return
        end
    end

	%---guardar los valores
	Registro('guardar','alias',alias);
	Registro('guardar','matrices',matrices);
	Registro('guardar','pantelides',pantelides);
    Registro('guardar','noedo',noedo);
    Registro('guardar','dirtrabajo',dirtrabajo);
    Registro('guardar','dirlibreria',dirlibreria);
    Registro('guardar','metodointegracion',metodointegracion);
    Registro('guardar','reltol',reltol);
    Registro('guardar','abstol',abstol);
    Registro('guardar','maxstep',maxstep);
    Registro('guardar','initialstep',initialstep);
   
    %---directorio base de moo
    baseDir=GetBaseDir;
    optionsFile=[baseDir '\gui\opcionesmoo.mat'];
    
    %---guardar el fichero de opciones
    save(optionsFile,'alias','matrices','pantelides','noedo','dirtrabajo','dirlibreria' ...
        ,'metodointegracion','reltol','abstol','maxstep','initialstep' ...
        )
end

%---cerrar cuadro de dilogo
delete(findobj('tag','IO'));

