%Descripcion:
%	Ejecuta la simulacin y muestra las variables a pintar
%03/08/03
function CBSimular
%declarar ROOT y ENTORNO
global ROOT ENTORNO

%limpia los ejes
CBLimpiar;

%recupera valores del registro
nombreBaseEdo=Registro('recuperar','nombreBaseEdo');
nombreEdo=Registro('recuperar','nombreEdo');
nombreNoEdo=Registro('recuperar','nombreNoEdo');
panel=Registro('recuperar','panel');
ruta=Registro('recuperar','ruta');

%preparar para las soluciones
ENTORNO.sol=[];
ENTORNO.t=[];

%leer el directorio actual
actualcd=cd;

%Leer los parametros, condiciones iniciales del panel
for i=1:length(panel)
   %leer el valor
   valor=str2num(get(panel(i).editor,'string'));
	
   %Si est vaco => mostrar un cuadro de dialogo informado de esto.
   if isempty(valor)
      mensaje=['La variable ' panel(i).nombre ' no tiene asignado valor'];
      errordlg(mensaje,'Error','modal');
      return;
   end

   %guardar el valor
   ENTORNO.infoVars(panel(i).lugar).valor=valor;	
end

%actualizar el tiempo
t=str2num(get(panel(end).editor,'string'));

%si solo es un punto el origen es 0
if length(t)==1
   %Si t<=0 => error
   if t<=0
      mensaje=['La variable time no puede ser negativa'];
      errordlg(mensaje,'Error','modal');
      return;
	end
   t=[0 t];
elseif length(t)==2
   %Si t(2)<t(1)  o t(1)<0 => error
   if t(1)<0
      mensaje=['El tiempo inicial no puede ser menor que cero'];
      errordlg(mensaje,'Error','modal');
      return;
   elseif t(2)<=t(1)
      mensaje=['El tiempo final no puede ser menor que tiempo inicial'];
      errordlg(mensaje,'Error','modal');
      return;
   end
else
   %nmero de elementos de t incorrecto
   mensaje=['time slo puede ser: tfinal o [tinicial tfinal]'];
   errordlg(mensaje,'Error','modal');
   return;
end   

%generar el valor de los parametros
iparametros=ObtenerIndiceVars(ENTORNO.infoVars,'parametro');
if isempty(iparametros)
   parametros=[];
else
   parametros=[ENTORNO.infoVars(iparametros).valor];
end

%generar el valor de las condiciones iniciales
iestados=ObtenerIndiceVars(ENTORNO.infoVars,'estado');
if isempty(iestados)
   inicial=[];
else
   inicial=[ENTORNO.infoVars(iestados).valor];
end

%Limpiar los ficheros de simulacion

%-----------------
%----SOLVE EDO----
%-----------------

% Check if there are any states
if isempty(inicial)
   puntosUnidad=Registro('recuperar','noedo');
   tiempo=[t(1):1/puntosUnidad:t(2)]';
   salidaEdo=[];
else
   cd (ruta);
   clear (nombreEdo);
   %try
      [tiempo,salidaEdo]=BaseEdo(t,inicial,parametros);
   %catch
   %   GestionErrores;
   %   return
   %end
end

%crear la estructura sol para las variables de la edo
for i=1:length(iestados)
   pos=length(ENTORNO.sol)+1;
   ENTORNO.sol(pos).variable=ENTORNO.infoVars(iestados(i)).expresion;
   ENTORNO.sol(pos).indice=iestados(i);
   ENTORNO.sol(pos).valor=salidaEdo(:,i);
end

%resolver las variables que no se calculan en la edo
cd (ruta);
clear (nombreNoEdo)
try
	salidaNoEdo=feval(nombreNoEdo,tiempo,salidaEdo,parametros); 
catch
   GestionErrores;
   return
end
%situarnos en el directorio de partida
%clear(nombreEdo);
%clear(nombreNoEdo);
cd (actualcd);

%crear la estructura sol para las variables de la no edo
idesconocidas=ObtenerIndiceVars(ENTORNO.infoVars,{'derivada' 'real'});
for i=1:length(idesconocidas)
   pos=length(ENTORNO.sol)+1;
   ENTORNO.sol(pos).variable=ENTORNO.infoVars(idesconocidas(i)).expresion;
   ENTORNO.sol(pos).indice=idesconocidas(i);
   ENTORNO.sol(pos).valor=salidaNoEdo(:,i);
end

%asignar valores a los alias
ceros=zeros(1,length(tiempo));
for i=1:length(ENTORNO.alias)
   ivars=ENTORNO.alias(i).ivars;
   isust=ENTORNO.alias(i).isust; %esta variable no tiene sentido cuando el alias es de tipo numero/derivada (de un parametro,
   										% nmero, tiempo)
   
   %dependiendo del tipo de alias hacer una cosa u otra
 	switch ENTORNO.alias(i).tipo  
   case 'parametro'
      %Todas las variables del alias son iguales al valor del parametro durante todo el tiempo
      valor=ENTORNO.infoVars(isust).valor+ceros;
      
      %calcular el ndice origen
      indOrigen=[2:length(ivars)];
   case 'numero' 
      valor=ENTORNO.alias(i).valor+ceros;
      
      %calcular el ndice origen
      indOrigen=[1:length(ivars)];      
   case 'conocida'
      % Variable time
      valor=tiempo;
      
      %calcular el ndice origen
      indOrigen=[2:length(ivars)];
   case {'real' 'estado'} 
      %las variables del alias toman el valor de la variable de sustitucion
      pos=find([ENTORNO.sol.indice]==isust);
      valor=ENTORNO.sol(pos).valor;
      
      %calcular el ndice origen
      indOrigen=[2:length(ivars)];
   case 'derivada'
      %el valor que toman las derivadas puede ser o de una variable o un nmero
      if isempty(isust)
         %Las derivadas toman valor constante
         valor=ENTORNO.alias(i).valor+ceros;
         
         %calcular el indice de origen
         indOrigen=[1:length(ivars)];
      else
         %hay variable de sustitucin => buscarla
         pos=find([ENTORNO.sol.indice]==isust);
         valor=ENTORNO.sol(pos).valor;	
         
         %calcular los indices de origen
         indOrigen=[2:length(ivars)];
      end
   end
      
   %crear los conjuntos de indices
   indDestino=length(ENTORNO.sol)+[1:length(indOrigen)];
   
   %copiar
   [ENTORNO.sol(indDestino).variable]=deal(ENTORNO.infoVars(ivars(indOrigen)).expresion);
   [ENTORNO.sol(indDestino).indice]=deal(ivars(indOrigen));
   [ENTORNO.sol(indDestino).valor]=deal(valor);
      
end

%pintar la lista de variables de la solucion
lista=findobj('tag','ISLista');

%poner el valor del elemento seleccionado a 1
set(lista,'value',1)

%poner la lista de variables ordenadas alfabticamente
[celdas,permutacion]=sort({ENTORNO.sol.variable});
ENTORNO.sol=ENTORNO.sol(permutacion);
set(lista,'string',celdas);

%guardar la solucin
ENTORNO.t=tiempo;