%Entrada:
%   iClase: indice de la clase
%	iComponente: indice del componente
%Descripcion:
%	Aplica la modificacion al componente iComponente de la clase iClase
%
function AplicarModificacion(iClase,iComponente)
global ROOT

%---calcular la modificacion del componente
AplicarModificacionSubcomponente(iClase,iComponente...
    ,ROOT.componente(iComponente).modificacion ...
    ,ROOT.componente(iComponente).nombre ...
    );

%---indicar que ya se ha aplicado la modificacion
ROOT.componente(iComponente).aplicadaModificacion=1;


%==========================================================================
%Entrada:
%	iClase: indice de la clase
%	iComponente: indice del componente
%   iModificacion: indice de la modificacion
%	nombreCompleto: nombre completo del componente dentro de la clase 
%                   incluyendo a iComponente
%Descripcion:
%	Aplica la modificacion a los subcomponentes de un componente
%
function AplicarModificacionSubcomponente(iClase,iComponente,iModificacion,raiz)
global ROOT

%---inicializar
%componente=ROOT.componente(iComponente);
modificacion=ROOT.modificacion(iModificacion);

%---calcular expresion
if ~isempty(modificacion.expresion)
    [r,t]=evaluar(iClase,modificacion.expresion,raiz);      
    AsignarValorComponente(iClase,iComponente,r,t,raiz);
end

%---recorrer los argumentos
for i=1:length(modificacion.argumento);
    %---obtener el argumento
    argumento=modificacion.argumento(i);
   
    %---obtener el nombre completo de la referencia
    if isempty(raiz)
        nombreReferencia=argumento.referencia;
    else
        nombreReferencia=[raiz '.' argumento.referencia];
    end
   
    %---buscar la referencia
    iReferencia=LookupReferenciaComponente(iComponente,argumento.referencia);
   
    %---Si no se encuentra => error
    if isempty(iReferencia)
        ROOT.merr=['$4$' ROOT.clase(iClase).nombre '$' nombreReferencia '$'];
        error(ROOT.merr);
    end
   
    %---aplicarle la modificacion
    AplicarModificacionSubcomponente(iClase,iReferencia ...
        ,argumento.modificacion,nombreReferencia);
end


%==========================================================================
%Entrada:
%   iClase: indice de la clase
%	iComponente: indice del componente
%	r: valor a almacenar
%	t: tipo de r
%   nombreCompleto: nombre completo del componente desde la clase
%
%Descripcion:
%	Asignar el valor a un componente de la siguiente manera
%		Si almacenado=='x' lo almacena en el subcampo value
%		Si almacenado=='b' | 'i' | 'r' | 's' lo almacena en valor
%
function AsignarValorComponente(iClase,iComponente,r,t,nombreCompleto)
global ROOT

%---inicializar
componente=ROOT.componente(iComponente);

%---averiguar donde se guarda
switch componente.almacenado
case 'x'
    %---buscar el subcomponente value y asignarle el valor de la expresion
    iGuardar=LookupComponenteEnComponente(iComponente,'value');
   
    %---modificar nombreCompleto
    nombreCompleto=[nombreCompleto '.value'];
   
    %---si no encontrado => error
    if isempty(iGuardar)
        ROOT.merr=['$4$' ROOT.clase(iClase).nombre '$' nombreCompleto '$'];
        error(ROOT.merr);
    end
otherwise
   iGuardar=iComponente;
end

%---obtener el componente donde se va a guardar el resultado
guardar=ROOT.componente(iGuardar);

%---obtener el tipo de iGuardar
switch ROOT.componente(iGuardar).almacenado
case 'b'
    tipo='Boolean';
case 'i'
    tipo='Integer';
case 'r'
    tipo='Real';
case 's'
    tipo='String';
end

%---obtener el tipo de r
switch t
case 'b'
    tipor='Boolean';
case 'i'
    if strcmp(tipo,'Real')
        tipor='Real';
    else
        tipor='Integer';
    end
case 'r'
    tipor='Real';
case 's'
    tipor='String';
end

%---ver si los tipos son compatibles
if ~strcmp(tipo,tipor)
    ROOT.merr=['$8$' ROOT.clase(iClase).nombre '$' nombreCompleto ...
        '$' tipo  '$' tipor];
    error(ROOT.merr);
end   

%---si ya se ha dado valor antes=> error
if guardar.aplicadaModificacion==0
    %---aplicar modificacion
  	ROOT.componente(iGuardar).valor=r;
    ROOT.componente(iGuardar).aplicadaModificacion=1;
else
    ROOT.merr=['$9$' ROOT.clase(iClase).nombre '$' nombreCompleto '$'];
    error(ROOT.merr);
end


%==========================================================================
%Descripcion:
%   Evala una expresin en una cadena
%
function [r,t]=evaluar(iClase,s,nombreCompleto)
%global ROOT

%---inicializar
token('open',s);

%---calcular
[r,t]=aritmetica(iClase,nombreCompleto);

%---cerrar
token('close');


%==========================================================================
%Descripcion:
%	averigua el tipo de una operacion. Los tipos pueden ser
%		'i' entero
%		'r' real
%
function t=promocion(t1,t2)
if t1=='r' || t2=='r'
    t='r';
else
    t='i';
end


%==========================================================================
%Sintaxis:
%
%Entrada:
%   iClase:
%   nombreCompleto:
%Salida:
%   r: valor de la expresion aritmetica
%	t: 'i' si el resultado es entero
%     'r' si el resultado es real
%
%Gramatica de una expresion aritmetica
%   aritmetica --> ['+' | '-' ] termino {('+' | '-') termino}
%	termino    --> factor {( '*' | '/') factor}
%	factor     --> primario ['^' primario]
%   primario   --> 'entero' | 'real' | '(' aritmetica ')' | funcion '(' 
%                  expresion ')'
%
function [r,t]=aritmetica(iClase,nombreCompleto)
%global ROOT

%---leer signo
[tk,s]=token;
switch tk
case 'menos'
    signo=-1;
case 'mas'
    signo=1;
otherwise
    token('unread',tk,s);
    signo=1;
end

%---leer un termino
[r,t]=termino(iClase,nombreCompleto);
r=signo*r;

%---leer {('+' | '-') termino}
while 1
    [tk,s]=token;
    switch tk
    case 'mas'
        [r2,t2]=termino(iClase,nombreCompleto);
        r=r+r2;
        t=promocion(t,t2);
    case 'menos'
        [r2,t2]=termino(iClase,nombreCompleto);
        r=r-r2;
        t=promocion(t,t2);
    otherwise
        token('unread',tk,s);
        return
    end
end


%==========================================================================
%
function [r,t]=termino(iClase,nombreCompleto)
global ROOT

%---leer factor
[r,t]=factor(iClase,nombreCompleto);

%---leer {('*' | '/') termino}
while 1
    [tk,s]=token;  
    switch tk
    case 'por'
        [r2,t2]=factor(iClase,nombreCompleto);
            
        %---calcular operacion
        r=r*r2;
        t=promocion(t,t2);
   case 'dividir'
        [r2,t2]=factor(iClase,nombreCompleto);
            
        %---calcular operacion
        if r2==0
            ROOT.merr=['$' ROOT.clase(iClase).nombre '$' nombreCompleto '$'];
            error(ROOT.merr);
        end
        r=r/r2;
        t=promocion(t,t2);
   otherwise
        token('unread',tk,s);
        return
   end
end


%==========================================================================
%
function [r,t]=factor(iClase,nombreCompleto)
%global ROOT

%---leer primario
[r,t]=primario(iClase,nombreCompleto);

%---leer ['^' primario]
[tk,s]=token;
if strcmp(tk,'potencia')
    [r2,t2]=primario(iClase,nombreCompleto);
    r=r^r2;
    t=promocion(t,t2);
else
    token('unread',tk,s);
end


%==========================================================================
%
function [r,t]=primario(iClase,nombreCompleto)
global ROOT

%---inicializar
r=[];
t='';

%---leer token
[tk,s]=token;
switch tk
case 'entero'
    r=str2double(s);
    t='i';
case 'real'
    r=str2double(s);
    t='r';
case 'name'
    %---buscar la clase y el componente
    [icl,ico,scl,sco]=LookupClaseYComponente(iClase,s);
   
    %---si no se encuentra la clase o el componente dar error
    if isempty(icl) || isempty(ico)
        ROOT.merr=['$4$' ROOT.clase(iClase).nombre '$' s '$'];
        error(ROOT.merr);
    end
   
    %---si no se ha hecho flattenning a la clase => hacerlo
    if ROOT.clase(icl).instanciada==0
        FlatteningClase(icl);
    end
   
    %si no se ha instanciado el componente => instanciarlo
    %if ROOT.componente(ico).instanciada<2
    %   InstanciarComponente(iClase,ico);
    %end
   
    %---Buscar sco en icl
    posicion=LookupReferenciaClase(icl,sco);
    if isempty(posicion)
        ROOT.merr=['$4$' ROOT.clase(icl).nombre '$' sco '$'];
        error(ROOT.merr);
    end
   
    %---Comprobar que sco tiene prefijo constante o parmetro
    if ~any(strcmp({'constant' 'parameter'},ROOT.componente(posicion).prefijo))
        ROOT.merr=['$11$' ROOT.Clase(iClase).nombre '$' nombreCompleto '$'];
        error(ROOT.merr);
    end
   
    %---Ver que tipo es sco
    switch ROOT.componente(posicion).almacenado
    case 'x'
        %---buscar el campo  value       
        iValue=LookupComponenteEnComponente(posicion,'value');
      
        %---si no existe el campo value dar error
        if isempty(iValue)
            ROOT.merr=['$4$' scl '$' sco '.value$'];
            error(ROOT.merr);
        end
      
        %---Si iValue es de tipo compuesto => error
        if ROOT.componente(iValue).almacenado=='x'
            ROOT.merr=['$12$' ROOT.clase(iClase).nombre '$' nombreCompleto '$'];
            error(ROOT.merr);
        else
            t=ROOT.componente(iValue).almacenado;
            r=ROOT.componente(iValue).valor;
        end

    otherwise
        %t=ROOT.componente(iValue).almacenado;
        %r=ROOT.componente(iValue).valor;
   end
   
case 'pabierto'
    %---leer aritmetica
    [r,t]=aritmetica;
   
    %---leer el )
    %[tk,s]=token;
    token;
   
case 'funcion'  
    %---leer el nombre de la funcion
    f=s;
   
    %---leer (
    %[tk,s]=token;
    token;
   
    %---leer aritmetica
    %[r,t]=aritmetica(iClase,nombreCompleto);
    r=aritmetica(iClase,nombreCompleto);
   
    %---leer )
    %[tk,s]=token;
    token;
   
    %---evaluar funcion
    r=feval(f,r);
    t='r';
end
 