%Entrada:
%	iclase: indice de la clase
%Descripcion:
%	Crea las ecuaciones de connect
function CrearConexiones(iClase)
%declarar ROOT
global ROOT

%ver si tiene clases de conector
for i=1:length(ROOT.clase(iClase).conector)
   %Obtener el conjunto de variables
   vars=ROOT.clase(iClase).conector{i};
   
   %Leer la primera variable
   primeraVar=vars{1};
   
   %Buscar donde est definida est variable
   iprimeraVar=Lookup2(iClase,primeraVar,'class','component','-');
   if isempty(iprimeraVar)
      ROOT.merr=['$4$' ROOT.clase(iClase).nombre '$' primeraVar '$'];
      error(ROOT.merr);
   end
   
   iespecificador=ROOT.componente(iprimeraVar).iespecificador;
   
   %ver que el especificador es de tipo connector
   if isempty(iespecificador) | ~strcmp(ROOT.clase(iespecificador).tipo,'connector')
      ROOT.merr=['$5$' ROOT.clase(iClase).nombre '$' especificador '$'];
      error(ROOT.merr);
   end
   
   %Obtener variables flow y continuas
   [flow,real]=ObtenerComponentesReales(iespecificador);
      
   %comprobar que los especificadores de las otras variables son compatibles
   for i=2:length(vars)
      %leer la variable
      var2=vars{i};
      
      %Buscar donde est definida esta variable
   	iVar2=Lookup2(iClase,var2,'class','component','-');
   	if isempty(iVar2)
         ROOT.merr=['$4$' ROOT.clase(iClase).nombre '$' var2 '$'];
	      error(ROOT.merr);
		end
   
   	%Obtener el especificador de esta variable
      iespecificador2=ROOT.componente(iVar2).iespecificador;
      
      %ver que el especificador es de tipo connector
   	if isempty(iespecificador) | ~strcmp(ROOT.clase(iespecificador2).tipo,'connector')
         ROOT.merr=['$5$' ROOT.clase(iClase).nombre '$' especificador2 '$'];
         error(ROOT.merr);
      end
      
      %Ver si este especificador es el mismo que el de la variable uno
      if iespecificador~=iespecificador2         
   		%Obtener variables flow y continuas
   		[flow2,real2]=ObtenerComponentesReales(iespecificador2);
         
         %Compararlas con flow y real
         resp=CompararListas(flow,flow2);
         if ~resp
            ROOT.merr=['$6$' ROOT.clase(iClase).nombre '$' especificador '$' especificador2 '$'];
            error(ROOT.merr);
        	end
         
         resp=CompararListas(real,real2);
         if ~resp
            ROOT.merr=['$6$' ROOT.clase(iClase).nombre '$' especificador '$' especificador2 '$'];
            error(ROOT.merr);
         end
      end
   end

   %generar las ecuaciones para conector{i}
   for j=1:length(flow)
      %crear ecuacion suma=0
      ecuacion=[vars{1} '.' flow{j}];
      for k=2:length(vars)
         ecuacion=[ecuacion '+' vars{k} '.' flow{j}];
      end
      ecuacion=[ecuacion '=0'];
      
      %guardar la ecuacion
      ROOT.clase(iClase).ecuacion{end+1}=ecuacion;
   end
   
   for j=1:length(real)
      %crear ecuacion v1=vn
      for k=2:length(vars)
         ecuacion=[vars{1} '.' real{j} '=' vars{k} '.' real{j}];
         
         %guardar la ecuacion
      	ROOT.clase(iClase).ecuacion{end+1}=ecuacion;
      end
   end     
end

%--------------------------------------------------------------------------
%Salida:
%	flow=lista de variables reales flow
%	real=lista de variables reales continuas
%Descripcion:
%	Obtiene dos listas de variables del especificador iespecificador. Una con los reales
%	otra con flow
function [flow,real]=ObtenerComponentesReales(iespecificador)
%declarar ROOT
global ROOT

%inicializar
flow={};
real={};

%empezar a buscar
for i=1:length(ROOT.clase(iespecificador).componente)
	%Obtener una referencia al componente
   icomponente=ROOT.clase(iespecificador).componente(i);
      
   %ver si el componente tiene como especificador Real
   if strcmp('Real',ROOT.componente(icomponente).especificador)
   	%ver su prefijo
      switch ROOT.componente(icomponente).prefijo
      case ''
      	%variable de igualacion
         real{end+1}=ROOT.componente(icomponente).nombre;
            
      case 'flow'
         %variable de suma
         flow{end+1}=ROOT.componente(icomponente).nombre;
      end
   end
end

