function ExtenderClase(iclase)
%declarar ROOT
global ROOT

%Si no hay extensiones => volver
if isempty(ROOT.clase(iclase).extension),return,end

%inicializar variables
nombreClase=ROOT.clase(iclase).nombre;
extension=ROOT.clase(iclase).extension;

%----------------------------
%--- RECORRER EXTENSIONES ---
%---  PARA REGISTRARLAS   ---
%----------------------------

%vreg consta de los camos:
%	vreg.nombre=[] nombre del componente
%	vreg.indices=[] vector de indices de los componente
%	vreg.sclase={} nombre de la clase del ndice
vreg=[];
for i=1:length(extension)
   %obtener el especificador de clase
   iespecificador=LookupEspecificador(iclase,extension(i).clase);
   if isempty(iespecificador)
      ROOT.merr=['$3$' nombreClase '$' extension(i).clase '$'];
      error(ROOT.merr);
   end
   ROOT.clase(iclase).extension(i).iclase=iespecificador;
   extension(i).iclase=iespecificador;
   
   %hacer flattening a la clase base
   FlatteningClase(iespecificador);
   
   %duplicar la clase base
   icomponentes=DuplicarComponentesClase(iespecificador,'');
   
   %aplicar modificaciones a lo nuevos componentes
   if ~isempty(extension(i).modificacion)
      %usar el componente ficticio uno creado en Parser
      ROOT.componente(1).hijo=icomponentes;
      ROOT.componente(1).modificacion=extension(i).modificacion;
      ROOT.componente(1).aplicadaModificacion=0;
   	AplicarModificacion(iclase,1)
   end   
      
   %registrar el nombre de los componentes
   for j=1:length(icomponentes)
      %Ver si ya est registrado el nombre
      if isempty(vreg)
         ind=[];
      else
      	ind=find(strcmp({vreg.nombre},ROOT.componente(icomponentes(j)).nombre));
      end   
         
      %Si no est registrado => registralo
      if isempty(ind)
         ind=length(vreg)+1;
         vreg(ind).nombre=ROOT.componente(icomponentes(j)).nombre;
         vreg(ind).indices(1)=icomponentes(j);
         vreg(ind).sclase{1}=extension(i).clase;
      else
         vreg(ind).indices(end+1)=icomponentes(j);
         vreg(ind).sclase{end+1}=extension(i).clase;
      end
   end
end

%---------------------------------
%--- GUARDAR LOS COMPONENTES   ---
%--- DE LAS EXTENSIONES EN LA  ---
%--- CLASE                     ---
%---------------------------------

vcomponente=ROOT.clase(iclase).componente;
lnombres={ROOT.componente(vcomponente).nombre};
for i=1:length(vreg)
   %leer el conjunto de indices
   indices=vreg(i).indices;
   
   %combinar componentes cuando hay ms de dos
   for j=2:length(indices)
      CombinarComponentesExtension(indices(j-1),indices(j),vreg(i).sclase(j-1),vreg(i).sclase(j));
   end
   
   %buscar si existe algn componente de la clase con el nombre del componente de la 
   %clase base
   ind=find(strcmp(lnombres,vreg(i).nombre));
   
   %si no existe => creamos uno nuevo
   if isempty(ind)
      ROOT.clase(iclase).componente(end+1)=indices(end);
      
      %borrar los componentes no necesarios
      %ROOT.componente(indices(1:end-1))=[];
   else
      %existe un componente en iclase ya con ese nombre
      CombinarComponentesExtension(indices(end),vcomponente(ind),vreg(i).sclase{end},ROOT.clase(iclase).nombre);
      
      %borrar los componentes no necesarios
      %ROOT.componente(indices(1:end-1))=[];
   end
end

%-----------------------------
%--- COPIAR LAS ECUACIONES ---
%-----------------------------
for i=1:length(extension)
   DuplicarEcuacionesEspecificador(extension(i).iclase,iclase,'');
end

%-------------------------------------------------------------------------------------------------------------------
%Entrada:
%	origen: indice a un componente
%	dest: indice de un componente
%	sorigen: nombre de la clase origen
%	sdest: nombre de la clase destino
%Descripcion:
%	combina los valores de dos componentes compatibles
%Nota:
%	El componente origen y destino tienen el mismo nombre
function CombinarComponentesExtension(origen,dest,sorigen,sdest)
%declarar ROOT
global ROOT

%obtener los hijos de los dos componentes
hijosOrigen=ROOT.componente(origen).hijo;
hijosDest=ROOT.componente(dest).hijo;

%si no tienen el mismo nmero de hijos => no son compatibles
if length(hijosOrigen)~=length(hijosDest)
   ROOT.merr=['$21$' sorigen '$' sdest '$' ROOT.componente(origen).nombre '$']; 
   error(ROOT.merr);
end

%combinar los valores
lTiposPredefinidos={'Boolean' 'Integer' 'Real' 'String'};
for i=1:length(hijosOrigen)
   %leer el prefijo de cada componente
   prefOrigen=ROOT.componente(hijosOrigen(i)).prefijo;
   prefDest=ROOT.componente(hijosDest(i)).prefijo;
   
   %si no son iguales => error
   if ~strcmp(prefOrigen,prefDest)
   	ROOT.merr=['$21$' sorigen '$' sdest '$' ROOT.componente(origen).nombre '$']; 
   	error(ROOT.merr);
   end
   
   %leer el especificador de cada componente
   tipoOrigen=find(strcmp(lTiposPredefinidos,ROOT.componente(hijosOrigen(i)).especificador));   
   tipoDest=find(strcmp(lTiposPredefinidos,ROOT.componente(hijosDest(i)).especificador));
   if isempty(tipoOrigen),tipoOrigen=0;,end
   if isempty(tipoDest),tipoDest=0;,end
   
   %si son distintos error
   if tipoOrigen~=tipoDest
   	ROOT.merr=['$21$' sorigen '$' sdest '$' ROOT.componente(origen).nombre '$']; 
   	error(ROOT.merr);
   end
   
   %ver que tipo es el especificador
   if tipoOrigen>0
      %de tipo predefinido
      CopiarValores(hijosOrigen(i),hijosDest(i));
   else
      %el especificador es una clase
      CombinarComponentesExtension(hijosOrigen(i),hijosDest(i),sorigen,sdest);
   end
end

%-----------------------------------------------------------------------------------------------------------------------
%Entrada:
%	origen: indice a componente de tipo predefinido
%	dest: indice a componente de tipo predefinido, del mismo tipo que origen
%Descripcion:
%	Copia los valores de origen en dest si el valor de destino es nan o ''
function CopiarValores(origen,dest)
%obtener los hijos de origen y destino
hijosOrigen=ROOT.componente(origen).hijos;
hijosDest=ROOT.componente(dest).hijos;

%buscar los atributos de dest que tienen como valor nan
ind=find(isnan(ROOT.componente(hijosDest).valor));
if ~isempty(ind)
   [ROOT.componente(hijosDest(ind)).valor]=deal(ROOT.componente(hijosOrigen(ind)).valor);
end

%buscar los atributos de dest que tienen como valor nan
ind=find(isnan(ROOT.componente(hijosDest).valor));
if ~isempty(ind)
   [ROOT.componente(hijosDest(ind)).valor]=deal(ROOT.componente(hijosOrigen(ind)).valor);
end


