%Descripcion:
%	Crea la funcin dx=f(t,x,p) que representa el edo
function ConstruirBaseEdo(nombreBaseEdo,nombreEdo,ruta)
%declarar ENTORNO
global ROOT ENTORNO
   
%Abrir el fichero base
fh=fopen([ruta '\' nombreBaseEdo '.m'],'w');
if fh==-1
   ROOT.merr=['$18$' ruta '\' nombreBaseEdo '.m' '$'];
   error(ROOT.merr);
end

%Leer opciones de integracion
metodos={'ode45' 'ode15s'};
metodointegracion=Registro('recuperar','metodointegracion');
if metodointegracion<0 | metodointegracion>2
   metodointegracion=1;
end

reltol=Registro('recuperar','reltol');
if reltol<0
   reltol=1e-3;
end
sOpciones=['''RelTol'',' num2str(reltol)];

abstol=Registro('recuperar','abstol');
if abstol<0
   abstol=1e-6;
end
sOpciones=[sOpciones ',''AbsTol'',' num2str(abstol)];

maxstep=Registro('recuperar','maxstep');
if ~isempty(maxstep)
   sOpciones=[sOpciones ',''MaxStep'',' num2str(maxstep)];
end

initialstep=Registro('recuperar','initialstep');
if ~isempty(initialstep)
   sOpciones=[sOpciones ',''InitialStep'',' num2str(initialstep)];
end

%ver si hay eventos
if ~isempty(ENTORNO.when)
   sOpciones=[sOpciones ',''Events'',''on'''];
end

%Crear la funcion
if ~isempty(ENTORNO.when)
   %hay eventos
   fprintf(fh,'function [tsol ,ysol]=%s(t,xe,p)\r\n',nombreBaseEdo);
   fprintf(fh,'%%---Opciones para resolver el edo\r\n');
   fprintf(fh,'opciones=odeset(%s);\r\n\r\n',sOpciones);
   
   fprintf(fh,'%%---Numero mximo de eventos que se van a tratar(para evitar cuelgues)\r\n');
   fprintf(fh,'contador=100;\r\n\r\n');
   
   fprintf(fh,'%%---Resolver edo\r\n');
   fprintf(fh,'tsol=[];\r\n');
   fprintf(fh,'ysol=[];\r\n');
   fprintf(fh,'tfinal=t(2);\r\n');
   
   %comiento del while
   fprintf(fh,'while 1\r\n');
   fprintf(fh,'	[t,y,te,xe,ie]=%s(''%s'',t,xe,opciones,p);\r\n\r\n',metodos{metodointegracion},nombreEdo);
   fprintf(fh,'	tsol=[tsol ; t];\r\n');
   fprintf(fh,'	ysol=[ysol ; y];\r\n\r\n');
   fprintf(fh,'%%---comprobar si ya hemos terminado de integrar\r\n');
   fprintf(fh,'	contador=contador-1;\r\n');
   fprintf(fh,'	if (t(end)==tfinal) | (contador<=0) \r\n');
   fprintf(fh,'		return\r\n');
   fprintf(fh,'	end\r\n\r\n'); 
   
   %fprintf(fh,'%%---Constantes\r\n');
   %iconstantes=ObtenerIndiceVars(ENTORNO.infoVars,'constante');
   %for i=1:length(iconstantes)
   %   var=ENTORNO.infoVars(iconstantes(i));
   %   fprintf(fh,'   %s=%s;\r\n',var.sustituida,num2str(var.valor));
   %end
   %fprintf(fh,'\r\n');
   
   fprintf(fh,'%%---parametros\r\n');
   iparametros=ObtenerIndiceVars(ENTORNO.infoVars,'parametro');
   for i=1:length(iparametros)
      var=ENTORNO.infoVars(iparametros(i));
		fprintf(fh,'	%s=p(%d);\r\n',var.sustituida,i);
   end
   fprintf(fh,'\r\n');   
   
   fprintf(fh,'%%---Tiempo y variables de estado\r\n');
   fprintf(fh,'   time=te;\r\n\r\n');
   
   fprintf(fh,'%%---Variables de estado\r\n');
   iestados=ObtenerIndiceVars(ENTORNO.infoVars,'estado');
   for i=1:length(iestados) 
      var=ENTORNO.infoVars(iestados(i));
      fprintf(fh,'	%s=xe(%d);\r\n',var.sustituida,i);
   end
   fprintf(fh,'\r\n'); 
   
   fprintf(fh,'%%---Llamar al DAE\r\n');
   fprintf(fh,'   [dxdt,alg]=%s(time,xe,''dae'',p);\r\n\r\n',nombreEdo);
   
   fprintf(fh,'%%---Derivadas de las variables de estado\r\n');
   for i=1:length(iestados) 
      var=ENTORNO.infoVars(iestados(i));
      fprintf(fh,'	d001%s=dxdt(%d);\r\n',var.sustituida,i);
   end
   fprintf(fh,'\r\n');
   
   fprintf(fh,'%%---variables algebraicas\r\n');
   ialgebraicas=ObtenerIndiceVars(ENTORNO.infoVars,'algebraica');
   for i=1:length(ialgebraicas) 
      var=ENTORNO.infoVars(ialgebraicas(i));
      fprintf(fh,'	%s=alg(%d);\r\n',var.sustituida,i);
   end
   fprintf(fh,'\r\n');
   
   %comienzo del switch
   fprintf(fh,'%%---Tratar los eventos\r\n');
   fprintf(fh,'	switch ie\r\n');
   numerocc=0; %almacena el nmero de cc de los eventos anteriores
   for i=1:length(ENTORNO.when)
      %escribir case
      tamanocc=length(ENTORNO.when(i).cc);  
      if tamanocc==1
         fprintf(fh,'	case %d\r\n',numerocc+1);
      else   
         fprintf(fh,'	case {%s}\r\n',int2str([numerocc+1:numerocc+tamanocc]));
         numerocc=numerocc+1;
      end
      numerocc=numerocc+tamanocc;
      
      %escribir reinit
      for j=1:length(ENTORNO.when(i).reinit);
         fprintf(fh,'		%s=%s;\r\n',ENTORNO.when(i).reinit(j).misustituida,ENTORNO.when(i).reinit(j).mdsustituida);
      end
	end
   fprintf(fh,'	end\r\n\r\n'); 
   %fin del switch
   
   fprintf(fh,'%%---Nuevo intervalo de integracin y nuevos valores para las variables de estado\r\n');
   fprintf(fh,'	t=[te tfinal];\r\n');
      
   fprintf(fh,'end\r\n'); 
   %fin del while
   
else
   %No hay eventos
   fprintf(fh,'function [tsol,ysol]=%s(t,xe,p)\r\n',nombreBaseEdo);
   fprintf(fh,'opciones=odeset(%s);\r\n',sOpciones);
   fprintf(fh,'[tsol,ysol]=%s(''%s'',t,xe,opciones,p);\r\n',metodos{metodointegracion},nombreEdo);
end

%Cerrar el fichero
fclose(fh);
