%Entrada:
%	ENTORNO
%	nombre= nombre de la funcion que se va a construir
%	ruta= directorio donde se creara la funcion terminado en '/'
%Salida:
%	error= cadena que describe el error que se ha producido
%Descripcion
%	construye una funcion que es una edo de primer orden a partir de los
%	bloques de un sistema.
%	La funcion tiene la estructura:
%	
%	function edo_dxdt=nombre(t,edo_x,edo_parametros)
%	variable de estado 1=edo_x(1);
%	...
%	variable de estado n=edo_x(n);
%	[salidas bloque 1]=fsolve('FnBloque1',0,optimset,entradas bloque 1);
%	...
%	[salidas bloque m]=fsolve('FnBloquem',0,optimset,entradas bloque m);
%	edo_dxdt(1)=primera derivada;
%	...
%	edo_dxdt(n)=derivada n-sima;
%Modificada 24/08/06 Antonio Belmonte
function ConstruirEdo(nombre,ruta)
%declarar ROOT ENTORNO
global ROOT ENTORNO
   
% Open file
fh=fopen([ruta '\' nombre '.m'],'w');
if fh==-1
   ROOT.merr=['$18$' ruta '\' nombre '.m' '$'];
   error(ROOT.merr);
end

%escribir la cabecera
if ~isempty(ENTORNO.when)
   %-----------------
   %---con eventos---
   %-----------------
   
   %case normal
   fprintf(fh,'function varargout=%s(time,x,flags,parametros)\r\n',nombre);
   fprintf(fh,'%%---Simulation file\r\n\r\n');
   fprintf(fh,'switch flags\r\n');
   fprintf(fh,'case ''''\r\n');
   fprintf(fh,'	varargout{1}=dae(time,x,parametros);\n');
   
   %case events
   fprintf(fh,'case ''events''\r\n');
   fprintf(fh,'	[varargout{1:3}]=fn_eventos(time,x,parametros);\r\n');
   
   %case moo
   fprintf(fh,'case ''dae''\r\n');
   fprintf(fh,'	[varargout{1:2}]=dae(time,x,parametros);\n');
   
   %fin switch
	fprintf(fh,'end\r\n\r\n');
   
   %aadir el cuerpo de la edo
   fprintf(fh,'%%----------------------------------------------\r\n');
   fprintf(fh,'function [dxdt,alg]=dae(time,x,parametros)\r\n');
   fprintf(fh,'%%---initial values for dxdt and alg\r\n');
   fprintf(fh,'dxdt=[];\n');
   fprintf(fh,'alg=[];\n');
   fprintf(fh,'\n');
   generarCuerpoEdo(fh,1);
   
   %aadir la funcin de eventos
   generarFunEventos(fh,nombre);
   
else
   %sin eventos
   fprintf(fh,'function dxdt=%s(time,x,flags,parametros)\r\n',nombre);
   fprintf(fh,'%%---Simulation file\r\n');
   generarCuerpoEdo(fh,0);
end

%cerrar el fichero 
fclose(fh);

%------------------------------------------------------------------------------------------------
%Entrada:
%	fh: file handler
%	eventos: 1 si se quiere tratamiento con eventos
%				0 si no se quiere tratamiento con eventos
%Descripcion:
%	Crear el cuerpo del edo
function generarCuerpoEdo(fh,eventos)
%declarar ENTORNO
global ENTORNO

%valores iniciales para empezar la simulacion de los bloques
variables='';
for i=1:length(ENTORNO.bloques)
   if strcmp(ENTORNO.bloques(i).tipo,'implicito')
      if isempty(variables)
         variables=['persistent control inicio' num2str(i)];
      else
         variables=[' inicio' num2str(i)];
      end
   end
end

if ~isempty(variables)
   %aadimos una variable control para ver si se ha inicializado las variables de iteracin
   fprintf(fh,'%s\r\n\r\n',variables);
   
   %indicar que hay algn bloque implicito
   hayImplicito=1;
else
   %indicar que no hay bloques implicitos
   hayImplicito=0;
end

%constantes
%fprintf(fh,'%%---Constantes\r\n');
%iconstantes=ObtenerIndiceVars(ENTORNO.infoVars,'constante');
%for i=1:length(iconstantes)
%   var=ENTORNO.infoVars(iconstantes(i));
%   fprintf(fh,'%s=%d;\r\n',var.sustituida,var.valor);
%end
%fprintf(fh,'\r\n');

% Parameters
fprintf(fh,'%%---Parmetros\r\n');
iparametros=ObtenerIndiceVars(ENTORNO.infoVars,'parametro');
for i=1:length(iparametros)
   var=ENTORNO.infoVars(iparametros(i));
   fprintf(fh,'%s=parametros(%d);\r\n',var.sustituida,i);
end     
fprintf(fh,'\r\n');

%obtener las variables de estado.
fprintf(fh,'%%---State variables\r\n');
iestados=ObtenerIndiceVars(ENTORNO.infoVars,'estado');
for i=1:length(iestados)
   var=ENTORNO.infoVars(iestados(i));
   fprintf(fh,'%s =x(%d);\r\n',var.sustituida,i);
end
fprintf(fh,'\r\n');

%escribir las ecuaciones
fprintf(fh,'%%---EQUATIONS\r\n');
for i=1:length(ENTORNO.bloques)
   if strcmp(ENTORNO.bloques(i).tipo,'explicito')
      for j=1:length(ENTORNO.bloques(i).sustituidas)
         fprintf(fh,'%s;\r\n',ENTORNO.bloques(i).sustituidas{j});
      end
      fprintf(fh,'\r\n');
   else      
      %Bloque es implicito, por tanto hay que despejar las variables
      fprintf(fh,'%%Implicit block\r\n');
      
      ientradas=ENTORNO.bloques(i).entradas;
      switch(length(ientradas))
      case 0
         fprintf(fh,'entradas=[];\r\n');
      case 1
         var=ENTORNO.infoVars(ientradas(1));
         fprintf(fh,'entradas=%s;\r\n',var.sustituida);
      otherwise
     		fprintf(fh,'entradas = zeros(1,%d);\r\n',length(ientradas));
         for j=1:length(ientradas)
            var=ENTORNO.infoVars(ientradas(j));  
         	fprintf(fh,'entradas(%d) = %s;\r\n',j,var.sustituida);   
         end
      end
      
      %seleccionar el punto de inicio
      tamanoBloque=length(ENTORNO.bloques(i).salidas);
      fprintf(fh,'if isempty(control)\r\n');
      fprintf(fh,'   inicio%d=zeros(1,%d);\r\n',i,tamanoBloque);
      fprintf(fh,'   inicio%d=valorInicial(''FnBloque%d'',%d,entradas,parametros);\r\n',i,i,tamanoBloque);
		fprintf(fh,'end\r\n');
      
      %resolver el bloque
      fprintf(fh,'inicio%d = hnewton_n(''FnBloque%d'',inicio%d,1e-5,%d,entradas,parametros);\r\n',i,i,i,tamanoBloque);
      
      %asignar una componente del  vector salida de fsolve a cada variable de salida del bloque
      isalidas=ENTORNO.bloques(i).salidas;
      for j=1:length(isalidas)
         var=ENTORNO.infoVars(isalidas(j));
      	fprintf(fh,'%s = inicio%d(%d);\r\n',var.sustituida,i,j);
      end
      fprintf(fh,'%%End of Implicit block \r\n\r\n');
   end
end
fprintf(fh,'%%---END OF EQUATIONS\r\n\r\n');

%generar ahora las salidas de la funcion edo
fprintf(fh,'%%---Output Derivatives\r\n');
for i=1:length(iestados)   
   %La variable de estado sustituida es de la forma rxxx, su derivada es d001rxxx
   var=['d001' ENTORNO.infoVars(iestados(i)).sustituida];
   fprintf(fh,'dxdt(%d,1) = %s;\r\n',i,var);
end
fprintf(fh,'\r\n');

%si estamos en eventos, devolver las variables algebraicas
if eventos
	ialgebraicas=ObtenerIndiceVars(ENTORNO.infoVars,'real');
	fprintf(fh,'%%---Output Algebraic Vars\r\n');
	for i=1:length(ialgebraicas)   
   	var=ENTORNO.infoVars(ialgebraicas(i)).sustituida;
   	fprintf(fh,'alg(%d,1) = %s;\r\n',i,var);
	end
	fprintf(fh,'\r\n');
end

%desactivar control si hay bloques implicitos
if hayImplicito
	fprintf(fh,'%%---Control de las condiciones de inicio\r\n');
	fprintf(fh,'control=1;\r\n');
	fprintf(fh,'\r\n');
end

%-------------------------------------------------------------------------------------------------
%Entrada:
%	fh: file handler
%	nombre: nombre de la funcion edo
%Descripcion:
%	Crea la funcin eventos
function generarFunEventos(fh,nombre)
%declarar ENTORNO
global ENTORNO

%cabera de la funcion
fprintf(fh,'%%-----------------------------------------------------------------------------------------\r\n');
fprintf (fh,'function [val,isterm,dir]=fn_eventos(time,x,parametros)\r\n');

%constantes
%fprintf(fh,'%%---Constantes\r\n');
%iconstantes=ObtenerIndiceVars(ENTORNO.infoVars,'constante');
%for i=1:length(iconstantes)
%   var=ENTORNO.infoVars(iconstantes(i)).sustituida;
%   valor=ENTORNO.infoVars(iconstantes(i)).valor;
%   fprintf(fh,'%s=%d;\r\n',var,valor);
%end
%fprintf(fh,'\r\n');

%Parameters
fprintf(fh,'%%---Parmetros\r\n');
iparametros=ObtenerIndiceVars(ENTORNO.infoVars,'parametro');
for i=1:length(iparametros)
   var=ENTORNO.infoVars(iparametros(i)).sustituida;
   fprintf(fh,'%s=parametros(%d);\r\n',var,i);
end     
fprintf(fh,'\r\n');

%obtener las variables de estado.
fprintf(fh,'%%---State variables\r\n');
iestados=ObtenerIndiceVars(ENTORNO.infoVars,'estado');
for i=1:length(iestados)
   var=ENTORNO.infoVars(iestados(i)).sustituida;
   fprintf(fh,'%s =x(%d);\r\n',var,i);
end
fprintf(fh,'\r\n');

%Llamar a la funcin dae
fprintf(fh,'%%---Call DAE\n');
fprintf(fh,'[dxdt,alg]=dae(time,x,parametros);\n\n');

%escribir las derivadas de los estados
fprintf(fh,'%%---Derivatives of state variables\n');
for i=1:length(iestados)
   var=['d001' ENTORNO.infoVars(iestados(i)).sustituida];
   fprintf(fh,'%s =dxdt(%d);\r\n',var,i);
end
fprintf(fh,'\r\n');

%escribir las variables algebraicas
fprintf(fh,'%%---Algebraic variables\n');
ialgebraicas=ObtenerIndiceVars(ENTORNO.infoVars,'real');
for i=1:length(ialgebraicas)
   var=ENTORNO.infoVars(ialgebraicas(i)).sustituida;
   fprintf(fh,'%s =alg(%d);\r\n',var,i);
end
fprintf(fh,'\r\n');

%crear los eventos
n=ENTORNO.whennumerocc;
if n==1
   fprintf(fh,'val=0;\r\n');
	fprintf(fh,'isterm=1;\r\n');
	fprintf(fh,'dir=1;\r\n');
else
	fprintf(fh,'val=zeros(1,%d);\r\n',n);
	fprintf(fh,'isterm=ones(1,%d);\r\n',n);
	fprintf(fh,'dir=ones(1,%d);\r\n',n);
end   

%escribir las condiciones
indcc=1;
for i=1:length(ENTORNO.when)
   for j=1:length(ENTORNO.when(i).cc)
      fprintf(fh,'val(%d)=%s;\r\n',indcc,ENTORNO.when(i).cc(j).sustituida);
      indcc=indcc+1;
   end
end
