%Entrada:
%	nombre= nombre de la funcion que se va a construir
%	ruta= directorio donde se creara la funcion terminado en '/'
%Descripcion:
%	Construye una funcion que es el sistema de las variables reales que 
%	no se calculan con el edo
function ConstruirNoEdo(nombre,ruta)
%declarar ROOT ENTORNO
global ROOT ENTORNO

% Open file
fh=fopen([ruta '\' nombre '.m'],'w');
if fh==-1
   ROOT.merr=['$18$' ruta '\' nombre '.m' '$'];
   error(ROOT.merr);
end

%escribir la cabecera
fprintf(fh,'function salida=%s(time,estados,parametros)\r\n',nombre);
fprintf(fh,'%%---Simulation file\r\n');

%Crear un vector de ceros
fprintf(fh,'z=zeros(length(time),1);\r\n');
fprintf(fh,'\r\n');

%escribir las constantes
%fprintf(fh,'%%---Constants\r\n');
%iconstantes=ObtenerIndiceVars(ENTORNO.infoVars,'constante');
%for i=1:length(iconstantes)
%   var=ENTORNO.infoVars(iconstantes(i));
%   fprintf(fh,'%s=%d;\r\n',var.sustituida,var.valor);
%end
%fprintf(fh,'\r\n');

%Parameters
fprintf(fh,'%%---Parameters\r\n');
iparametros=ObtenerIndiceVars(ENTORNO.infoVars,'parametro');
for i=1:length(iparametros)
   var=ENTORNO.infoVars(iparametros(i));
   fprintf(fh,'%s=parametros(%d);\r\n',var.sustituida,i);
end
fprintf(fh,'\r\n');

%entradas
fprintf(fh,'%%---State vars\r\n');
iestados=ObtenerIndiceVars(ENTORNO.infoVars,'estado');
for i=1:length(iestados)
   var=ENTORNO.infoVars(iestados(i));
   fprintf(fh,'%s=estados(:,%d);\r\n',var.sustituida,i);
end
fprintf(fh,'\r\n');

%Escribir las ecuaciones
fprintf(fh,'%%---EQUATIONS\r\n');

for i=1:length(ENTORNO.bloques)
   if strcmp(ENTORNO.bloques(i).tipo,'explicito')
      %Bloque Explicito
      for j=1:length(ENTORNO.bloques(i).sustituidas)
         fprintf(fh,'%s;\r\n',Vectorizar(ENTORNO.bloques(i).sustituidas{j}));
      end
      fprintf(fh,'\r\n');
   else      
      %Bloque implicito
      fprintf(fh,'%%---Implicit block\r\n');
      
      ientradas=ENTORNO.bloques(i).entradas;
      switch(length(ientradas))
      case 0
         %no hacer nada
      case 1
         var=ENTORNO.infoVars(ientradas(1));
         fprintf(fh,'entradas=%s;\r\n',var.sustituida);
      otherwise
     		fprintf(fh,'entradas = zeros(1,%d);\r\n',length(ientradas));
         for j=1:length(ientradas)
         	var=ENTORNO.infoVars(ientradas(i));   
         	fprintf(fh,'entradas(:,%d) = %s;\r\n',j,var.sustituida);
         end
      end
      
      %Resolver el bloque implicito. Tiene un tamao igual al nmero de salidas
      isalidas=ENTORNO.bloques(i).salidas;
      if ~length(ientradas)
         %No hay entradas. Por tanto el bloque se resuelve una sola vez
         fprintf(fh,'inicio=valorInicial(''FnBloque%d'',%d,[],parametros);\r\n',i,length(isalidas));
         fprintf(fh,'inicio = hnewton_n(''FnBloque%d'',inicio,1e-5,%d,[],parametros);\r\n',i,length(isalidas));
         
         %variable de salida del bloque
      	for j=1:length(isalidas)
         	var=ENTORNO.infoVars(isalidas(j));
         	fprintf(fh,'%s = inicio(%d)+z;\r\n',var.sustituida,j);
      	end
      else
         %Hay entradas. Resolver para cada valor de time
      	fprintf(fh,'inicio=valorInicial(''FnBloque%d'',%d,entradas(1,:),parametros);\r\n',i,length(isalidas));
      	fprintf(fh,'for i=1:length(time)\r\n');
      	fprintf(fh,'   inicio = hnewton_n(''FnBloque%d'',inicio,1e-5,%d,entradas(i,:),parametros);\r\n',i,length(isalidas));
      
      	%variable de salida del bloque
      	for j=1:length(isalidas)
         	var=ENTORNO.infoVars(isalidas(j));
         	fprintf(fh,'   %s(i,1) = inicio(%d);\r\n',var.sustituida,j);
      	end
         fprintf(fh,'end\r\n');
      end
      
      fprintf(fh,'%%---End of Implicit block\r\n\r\n');
   end
end
fprintf(fh,'%%---END OF EQUATIONS\r\n\r\n' );

%salidas
fprintf(fh,'%%---Outputs\r\n');
idesconocidas=ObtenerIndiceVars(ENTORNO.infoVars,{'derivada' 'real'});
for i=1:length(idesconocidas)
   var=ENTORNO.infoVars(idesconocidas(i));
   fprintf(fh,'salida(:,%d)=%s;\r\n',i,var.sustituida);
end

%cerrar el fichero
fclose(fh);

%----------------------------------------------------------------------------------
%Entrada:
%	f=expresion que contiene la formula a vectorizar
%Salida
%	f=expresion con la formula vectorizada
%Descripcion:
%	Convierte una formula en su equivalente vectorizada
function f=Vectorizar(f)
f=strrep(f,'*','.*');
f=strrep(f,'/','./');
f=strrep(f,'^','.^');

%esta expresion es para convertir las constantes en vectores
f=[f '+z'];