%Entrada:
%	ecuacion: cadena que contiene la ecuacion
%	varInd: nombre de la variable independiente
%	lista: lista de variables sobre las que se va a derivar
function solucion=Derivadar(ecuacion,varInd,lista)
%buscar el =
pos=find(ecuacion=='=');

%obtener el miembro izqdo y derecho de la ecuacion
mizqdo=ecuacion(1:pos-1);
mdrcho=ecuacion(pos+1:end);

%derivar los dos miembros
mdrcho=DerivarMiembro(mdrcho,varInd,lista);
mizqdo=DerivarMiembro(mizqdo,varInd,lista);

%devolver solucion
solucion=[mizqdo '=' mdrcho];

%-----------------------------------------------------------------------------------------------
%Entrada:
%Salida:
%Descripcion:
function solucion=DerivarMiembro(expresion,varInd,lista)
%derivar respecto la variable independiente
if ~isempty(varInd)
%   derivada=char(diff(expresion,varInd));
   derivada=char(diff(str2sym(expresion),varInd));
   
   %simplificamos un poco la expresin
   switch (derivada)
   case '0'
      %no grabar nada
      solucion='';
   otherwise
      %compiar el resultado
      solucion=derivada;
   end
else
   solucion='';
end

%derivadar respecto las variables de lista
for i=1:length(lista)
   %obtener la variable
   var=lista{i};
   
   %derivar
   % derivada=char(diff(expresion,var));
   derivada=char(diff(str2sym(expresion),var));
   
   %Ver si la variable es real o derivada
   if var(1)=='d'
      %es una derivada
      orden=str2num(var(2:4));
      prefijo=['000' num2str(orden+1)];
      vard=['d' prefijo(end-2:end) var(5:end)];
   else
      %es una variable real
      vard=['d001' var];
   end
   
   %simplificar un poco la expresion
   switch(derivada)
   case '0'
      %no hacer nada
      s='';
   case '1'
      if isempty(solucion)
         s=vard;
      else
         s=['+' vard];
      end
   case '-1'
      s=['-' vard];
   otherwise
      s=EscribirDerivada(derivada,vard);
      
      %ver si ya hay otros factores
      if s(1)~='-' & ~isempty(solucion)
         s=['+' s];
      end
      
   end
   
   %escribir la derivada
   solucion(end+1:end+length(s))=s;      
end

%comprobaciones finales
if isempty(solucion)
   %devolver cero si la solucion es vacia
   solucion='0';
end
  
%------------------------------------------------------------------------------
%Entrada:
%		expresion: cadena
%Salida:
%		resp=1 si cadena tiene signos + o -. 
%			  0 si no los tiene o empieza y acaba en parntesis
function resp=EsCompuesta(expresion)
%Si el primer signo es '+' 0 '-' no se tiene en cuenta
if expresion(1)=='-' | expresion(1)=='+'
   token('open',expresion(2:end));
else
   token('open',expresion);
end
 
%procesar la cadena
resp=0;
parentesis=0;
[tk,s]=token;
while ~strcmp(tk,'fin')
   switch tk
   case '('
      parentesis=parentesis+1;
   case ')'
      parentesis=parentesis-1;
   case {'mas' 'menos'}
      if ~parentesis
         resp=1;
         break;
      end
   end
    
   %siguiente token
   [tk,s]=token;
end
 
token('close');
          
%-------------------------------------------------------------------------------------
%Entrada:
%	expresion: expresion de la derivada
%	vard:	variable derivada
%Salida:
%	solucion: expresion de la derivada con vard ya incluido
%Descripcion:
%	Construye la expresion de la derivada, teniendo en cuenta la siguiente simplificacion:
%		expresion =['-'] vard ['^' numero] => [ '-'] vard ['^' numero+1]
function solucion=EscribirDerivada(expresion,vard)
if EsCompuesta(expresion)
   solucion=['(' expresion ')*' vard];
   return
else
   %procesar la cadena
   token('open',expresion);
    
   %ver el signo y la variable
   [tk,s]=token;
   if strcmp(tk,'menos')   
      signo=s;
      %leer la variable
      [tk,s]=token
      if strcmp(tk,'name')
         variable=s;
      else
         %devolver la derivada
         solucion=[expresion '*' vard];
         token('close');
       	return   
      end
    elseif strcmp(tk,'name')
       variable=s;
    else
       %devolver la derivada
       solucion=[expresion '*' vard];
       token('close');
       return
    end
    
    %Si las variables no son iguales => volver
    if ~strcmp(vard,variable)
       %devolver la derivada
       solucion=[expresion '*' vard];
       token('close');
       return
    end
    
    %ver si ahora viene ^
    [tk,s]=token;
    if strcmp(tk,'potencia')
       %ahora se lee un nmero
    	[tk,s]=token;
    	if strcmp(tk,'entero') | strcmp(tk,'real') 
      	 numero=str2num(s);
    	else
      	%devolver la derivada
       	solucion=[expresion '*' vard];
       	token('close');
       	return
    	end
    
    	%ahora leer fin
    	[tk,s]=token;
    	if ~strcmp(tk,'fin')
       	%devolver la derivada
       	solucion=[expresion '*' vard];
       	token('close');   
       	return
    	end
    elseif strcmp(tk,'fin')
       numero=1;
    else
       %devolver la derivada
       solucion=[expresion '*' vard];
       token('close');
       return
    end
    
    %construir la expresion para la derivada
    solucion=[signo vard '^' num2str(numero+1)];
    token('close');   
    return
 end
 