%Entrada:
%	iclase: indice a la clase. Slo se utiliza para obtener el nombre de la clase
%Descripcion:
%	Eliminar los alias de un conjunto de ecuaciones
%	Crea los conjuntos de alias con la siguiente estructura
%		ENTORNO.alias(i).ivars=[indice en infoVars a las variables del conjunto]
%							 .lvars1={lista de variables del alias}
%							 .lvars2={lista de variables del alias sustituidas}
%							 .tipo='libre' | 'tiempo' | 'parametro' | 'estado' | 'numero'
%							 .isust= indice en infoVars de la variable de sustitucion
%							 .sust1= nombre de la variable de sustitucion
%							 .sust2= nombre de la variable de sustitucion sustituida
%							 .valor= valor nmerico de la variable para 'parametro' 'estado' 'nmero'
%Modificado 12/09/09 Antonio Belmonte

%Hay un bug:
%	si pasamos el conjunto de ecuaciones
%		declare
%			constante p=7
%			parametro q=8
%			real u,v,w
%		equations
%			u=p
%			v=w
%			w=q
%		end
%Quita todas las ecuaciones, con lo cual no se puede simular!!!!
function EliminarAlias
%declarar ENTORNO
global ENTORNO

%comprobar si el usuario que eliminar alias
if ~Registro('recuperar','alias'),return,end

%Primero encontrar los alias y formar su conjunto
ecsIntermedias=[];
for i=1:length(ENTORNO.ecuaciones)
   [resp,ivar1,valor1,tipo1,ivar2,valor2,tipo2]=EsAlias(i);
   %Si resp es 0 no es una alias
   %Si resp es 1 es que es un alias de la forma:
   %	variable=variable
   %	variable=nmero
   %	nmero=variable
   %Si resp es 2 es una ecuacin de la forma:
   %	nmero=nmero => se elimina la ecuacin
   switch resp
   case 0
      %aadir la ecuacin a ecuaciones intermedias sin ninguna modificacin
      ecsIntermedias(end+1)=i;
   case 1
      %%%%%%%%%%%%%%%%%%%
      %%% es un alias %%% => suprimir la ecuacion.
      %%%%%%%%%%%%%%%%%%%
      
      %Buscar si existen ya las variables en un alias
      if ~strcmp(tipo1,'numero')
         ind1=BuscarAlias(ivar1);
      else
         ind1=[];
      end
      
      if ~strcmp(tipo2,'numero')
         ind2=BuscarAlias(ivar2);
      else
         ind2=[];
      end
      
      switch 2*isempty(ind1)+isempty(ind2)
      case 0 
         %ind1 no vacio, ind2 no vacio
         UnirAlias(ind1,ind2);
      case 1
         %ind1 no vacio, ind2 vacio
         AgregarAlias(ind1,ivar2,valor2,tipo2);
      case 2
         %ind1 vacio, ind2 no vacio
         AgregarAlias(ind2,ivar1,valor1,tipo1);
      case 3
         % ind1 vacio y ind2 vacio
         CrearAlias(ivar1,valor1,tipo1,ivar2,valor2,tipo2);
      end     
      
   case 2
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % ecuacion de la forma num=num %%% => no aadirla
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   end   
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Si no hay alias => volver   %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isempty(ENTORNO.alias),return,end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%*** Si v y w son estados y alias => der(v) y der(w) pasan a ser alias %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
iestados=ObtenerIndiceVars(ENTORNO.infoVars,'estado');
aliasReal=find(strcmp({ENTORNO.alias.tipo},'real'));
aliasInvestigar=[1:length(ENTORNO.alias)];
aliasInvestigar(aliasReal)=[];
for i=aliasInvestigar
   %averiguar cuntas variables de estado tiene
   ievars=sort([iestados ENTORNO.alias(i).ivars]);
   J=find(ievars(1:end-1)==ievars(2:end));
   ievars=ievars(J);
   
   %decidir segn el tipo de alias
   ivars=[];
   tipo=ENTORNO.alias(i).tipo;
   switch tipo
   case {'parametro' 'numero' 'conocida'}
      %si hay al menos una derivada
      if length(ievars)>0
         %Obtener el ndice de las variables derivadas
         nombre={};
      	for j=ievars
         	nombre{end+1}=['der(' ENTORNO.infoVars(j).expresion ')'];
      	end
      	ivars=BuscarVarInfo(ENTORNO.infoVars,nombre);
         
         %No tiene variable de sustitucion
         isust=[];
         if strcmp(tipo,'conocida')
            valor=1;
         else
            valor=0;
         end
      end
      
    case 'estado'     
       %si hay ms de una variable en idvars, construir un nuevo conjunto de alias
   	if length(ievars)>1
         %Obtener el ndice de las variables derivadas
         nombre={};
      	for j=ievars
         	nombre{end+1}=['der(' ENTORNO.infoVars(j).expresion ')'];
      	end
      	ivars=BuscarVarInfo(ENTORNO.infoVars,nombre);
   
   		%la variable de sustitucin es la primera
         isust=ivars(1);
         valor=nan;
      end
   end
         
   %crear el nuevo alias si hay variables
   if ~isempty(ivars)
      pos=length(ENTORNO.alias)+1;
      ENTORNO.alias(pos).ivars=ivars;
      ENTORNO.alias(pos).tipo='derivada';
      ENTORNO.alias(pos).lvars1={ENTORNO.infoVars(ivars).expresion};
      ENTORNO.alias(pos).lvars2={ENTORNO.infoVars(ivars).sustituida};
      ENTORNO.alias(pos).isust=isust;
      if ~isempty(isust)
      	ENTORNO.alias(pos).sust1=ENTORNO.infoVars(isust).expresion;
         ENTORNO.alias(pos).sust2=ENTORNO.infoVars(isust).sustituida;
      else
         ENTORNO.alias(pos).sust1='';
         ENTORNO.alias(pos).sust2='';
      end
      ENTORNO.alias(pos).valor=valor;
   end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% construir el nuevo sistema de ecuaciones sin alias %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Para ello vamos recorriendo las ecuaciones intermedias y 
%sustituyendo una a una las variables alias. 
%A continuacin hacemos lo mismo con ecuacionesIntermedias(i).variables{j}
for i=1:length(ecsIntermedias)
   %vamos a leer la ecuacion token a token
   cadena='';
   sustituida='';
   token('open',ENTORNO.ecuaciones(ecsIntermedias(i)).expresion);
   
   %procesar la ecuacion
   ivars=[];	%indice de las variables a infoVars
   lvars1={};	%lista de variables
   lvars2={};	%lista de variables sustituidas
   
   [tk,s]=token;
   while ~strcmp(tk,'fin')
      %buscar la variable en los alias
      switch tk
      case 'name'
         %buscar s en infoVars
         ivar=BuscarVarInfo(ENTORNO.infoVars,s);
         
         %obtener la variable de alias
         posicion=BuscarAlias(ivar);
         if ~isempty(posicion)
            %si el alias es numrico sustituir por el nmero
            if strcmp(ENTORNO.alias(posicion).tipo,'numero')
               ivar=[];
               sust1=num2str(ENTORNO.alias(posicion).valor);
               sust2=sust1;
            else
            	%cambiar ivar por su alias
               ivar=ENTORNO.alias(posicion).isust;
            end
         end   
         
         %obtener el nombre de la var
         if ~isempty(ivar)
         	var1=ENTORNO.infoVars(ivar).expresion;   
            var2=ENTORNO.infoVars(ivar).sustituida;
            sust1=var1;
            sust2=var2;
            
            %agregamos la variable var a ivars y lvars
         	if isempty(BuscarVarVector(ivars,ivar))
            	ivars(end+1)=ivar;
            	lvars1{end+1}=var1;
            	lvars2{end+1}=var2;
         	end
         end
            
         %guardar en la ecuacion las sustituciones
         cadena(end+1:end+length(sust1))=sust1;
         sustituida(end+1:end+length(sust2))=sust2;
         
      case 'der'
         %leer parntesis (
         [tk,s]=token;
         
         %leer el name
         [tk,s]=token;
                  
         %buscar la variable s 
         ivar=BuscarVarInfo(ENTORNO.infoVars,s);
                  
         %obtener la variable de alias para s. 
         posicion=BuscarAlias(ivar);
         if ~isempty(posicion)
            if any(strcmp(ENTORNO.alias(posicion).tipo,{'numero' 'parametro'}))
               %si el alias es numrico, parmetro sustituir por 0
               ivar=[];
               sust1='0';
               sust2='0';
            elseif strcmp(ENTORNO.alias(posicion).tipo,'conocida');
               %%si el alias es tiempo, parmetro sustituir por 1
               ivar=[];
               sust1='1';
               sust2='1';
            else
               ivar=ENTORNO.alias(posicion).isust;
            end   
         end
         
         %obtener las sutituciones
         if ~isempty(ivar)
            var1=['der(' ENTORNO.infoVars(ivar).expresion ')'];
            var2=['d001' ENTORNO.infoVars(ivar).sustituida];
            sust1=var1;
            sust2=var2;
                        
            %agregamos la variable var derivada a lvars1 y lvars2
            ivar=BuscarVarInfo(ENTORNO.infoVars,var1);
         	if isempty(BuscarVarVector(ivars,ivar))
            	ivars(end+1)=ivar;
            	lvars1{end+1}=var1;
            	lvars2{end+1}=var2;
         	end
         end
            
         %guardar en la ecuacion las sustituciones
         cadena(end+1:end+length(sust1))=sust1;
         sustituida(end+1:end+length(sust2))=sust2;
         
         %leer parntesis )
         [tk,s]=token;
         
      otherwise
         %dejar este elemento intacto
         cadena(end+1:end+length(s))=s;
         sustituida(end+1:end+length(s))=s;
      end
      
		%leer el siguiente token      
      [tk,s]=token;
   end
   
   %ahora, ya tenemos la nueva ecuacion. Almacenarla
   ENTORNO.ecuaciones(i).expresion=cadena;
   ENTORNO.ecuaciones(i).sustituida=sustituida;
   ENTORNO.ecuaciones(i).vars=ivars;
   ENTORNO.ecuaciones(i).lvars1=lvars1;
   ENTORNO.ecuaciones(i).lvars2=lvars2;
end   

%borrar las ecuaciones que sobran
numecs=length(ecsIntermedias);
if numecs<length(ENTORNO.ecuaciones)
	ENTORNO.ecuaciones(numecs+1:end)=[];
end

%modificar ENTORNO.infoVars para ajustar las variables de alias
for i=1:length(ENTORNO.alias)
   %quitar la variable de sustitucion
   if ~isempty(ENTORNO.alias(i).isust)
      %hay variable de sustitucion
   	v=ENTORNO.alias(i).ivars([2:end]);
   else
      %no hay variable de sustitucion pq es un nmero
      v=ENTORNO.alias(i).ivars;
   end
      
   %cambiar su tipo
   [ENTORNO.infoVars(v).tipo]=deal('alias');
end
         
%----------------------------------------------------------------------------------
%Entrada:
%	iec=indice de la ecuacin en ENTORNO.ecuacion
%Salida:
%	resp=0 => la ecuacin no es una alias 
%	resp=1 => la ecuacin es un alias
%				 .) si v=w => ivar1=indice(v), tipo1=tipo(v), ivar2=indice(w), tipo2=tipo(w)
%				 .) si v=num => ivar1=indice(v), tipo1=tipo(v), ivar2=num, tipo2='numero'
%				 .) si num=v => ivar1=indice(v), tipo1=tipo(v), ivar2=num, tipo2='numero'
% resp=2 => la ecuacin es de la forma:
%				num1=num2 => Si num1 y num2 son distintos da error
%Descripcion:
%	Averigua si ecuacion es un alias 
function [resp,ivar1,valor1,tipo1,ivar2,valor2,tipo2]=EsAlias(iec)
%declarar ROOT ENTORNO
global ROOT ENTORNO

%inicializar
ivar1=[];
valor1=nan;
tipo1='';
ivar2=[];
valor2=nan;
tipo2='';

%Inicializar token
token('open',ENTORNO.ecuaciones(iec).expresion);

%procesar la entrada
estado=1;
while 1
   %leer un token
   [tk,s]=token;
      
   %Diagrama de estados
   switch estado
   case 1
      if strcmp(tk,'name')
         %Buscar la variable en infoVars
         ivar1=BuscarVarInfo(ENTORNO.infoVars,s);
         if strcmp(ENTORNO.infoVars(ivar1).tipo,'estado')
            valor1=ENTORNO.infoVars(ivar1).start;
         else
            valor1=ENTORNO.infoVars(ivar1).valor;
			end
         tipo1=ENTORNO.infoVars(ivar1).tipo;         
         estado=2;
      elseif any(strcmp({'entero' 'real'},tk))
         ivar1=[];
         valor1=str2double(s);
         tipo1='numero';
         estado=5;
      else 
         %fallo
         resp=0;
         break
      end
      
   case 2
      if strcmp(tk,'igual')
         estado=3;
      else
         %fallo
         resp=0;
         break
      end
	
	case 3
      if strcmp(tk,'name')
   		%Buscar la variable en infoVars
         ivar2=BuscarVarInfo(ENTORNO.infoVars,s);
         if strcmp(ENTORNO.infoVars(ivar2).tipo,'estado')
            valor1=ENTORNO.infoVars(ivar2).start;
         else
            valor1=ENTORNO.infoVars(ivar2).valor;
			end
         tipo2=ENTORNO.infoVars(ivar2).tipo;      
         estado=4;
		elseif any(strcmp({'entero' 'real'},tk))
         ivar2=[];
         valor2=str2double(s);
         tipo2='numero';
         estado=4;
      else
         %fallo
         resp=0;
         break
      end      
      
   case 4
      if strcmp(tk,'fin')
         %exito
         resp=1;
         break
      else
         %fallo
         resp=0;
         break
      end
      
   case 5
      if strcmp(tk,'igual')
         estado=6;
      else   
         %fallo
         resp=0;
         break
      end
	
	case 6
      if strcmp(tk,'name')
         %copiar el nmero en ivar2 y tipo2
         ivar2=ivar1;
         valo2=valor1;
         tipo2=tipo1;
         %Buscar la variable
         ivar1=BuscarInfoVar(ENTORNO.infoVars,s);
         if strcmp(ENTORNO.infoVars(ivar1).tipo,'estado')
            valor1=ENTORNO.infoVars(ivar1).start;
         else
            valor1=ENTORNO.infoVars(ivar1).valor;
			end
         tipo1=ENTORNO.infoVars(ivar1).tipo;
         estado=4;
      elseif any(strcmp({'entero' 'real'},tk))
         ivar2=[];
         valor2=str2double(s);
         tipo2='numero';
         estado=7;
      else
			%fallo
         resp=0;
        	break         
		end
      
   case 7
      if strcmp(tk,'fin')
         %Si valor1 y valor2 son distintos => error
         if valor1~=valor2
            nombreClase=ROOT.clase(ROOT.iclase).nombre;
            ROOT.merr=['$23$' nombreClase '$' ENTORNO.ecuaciones(iec).expresion '$'];
            error(ROOT.merr);
         end
         
         %exito
         resp=2;
         break
      else
         %fallo
         resp=0;
         break;
      end
   end
end

%cerrar la cadena
token('close');

%-------------------------------------------------------------------------------
%Salida:
%	resp= 0 incompatibles
%		   1 compatibles y el tipo resultante es de tipo1
%			2 compatibles y el tipo resultante es de tipo2
%	tipo= tipo resultante
%function [resp,ivar,valor,tipo]=SonIncompatibles(ivar1,valor1,tipo1,ivar2,valor2,tipo2)
function resp=SonCompatibles(ivar1,valor1,tipo1,ivar2,valor2,tipo2)
%declarar ROOT y ENTORNO
global ROOT ENTORNO

%inicializar 
lista={'real' 'conocida' 'parametro' 'estado' 'numero'};
incompatibilidades=[0 0 0 0 0; 0 0 1 2 1; 0 1 3 4 1; 0 2 4 5 6; 0 1 1 6 7];
respuesta=[1 2 2 2 2; 1 1 0 1 0; 1 0 1 1 0; 1 2 2 1 2; 1 0 0 1 1];

%buscar las incompatibilidades entre tipo1 y tipo2
ind1=find(strcmp(lista,tipo1));
ind2=find(strcmp(lista,tipo2));
v=incompatibilidades(ind1,ind2);
resp=respuesta(ind1,ind2);

%decidir segn el tipo de incompatibilidad
switch v
case 0
   %sin error
   return
case 1
   %error. (numero,tiempo), (parametro,tiempo), (parametro,nmero)
   if strcmp(tipo1,'numero')
      s1=num2str(valor1);
   else
      s1=ENTORNO.infoVars(ivar1).expresion;
   end
   
   if strcmp(tipo2,'numero')
      s2=num2str(valor2);
   else
      s2=ENTORNO.infoVars(ivar2).expresion;
   end
   
   nombreClase=ROOT.clase(ROOT.iclase).nombre;
   ROOT.merr=['$24$' nombreClase '$' s1 '$' s2 '$'];
   error(ROOT.merr);
   
case 2
   %sin error. (FUTURO warning) (tiempo,estado)
   return
   
case 3
   %(parametro,parametro)
   %si tienen valores nmericos => error
   s1=ENTORNO.infoVars(ivar1).expresion;
   s2=ENTORNO.infoVars(ivar2).expresion;
      
   if valor1~=valor2
      nombreClase=ROOT.clase(ROOT.iclase).nombre;
   	ROOT.merr=['$24$' nombreClase '$' s1 '$' s2 '$'];
   	error(ROOT.merr);
   else
   	%aceptar
   	return
   end
   
case 4
   %(estado,parametro)
   %(FUTURO dar warning, ahora aceptar sin ms)
   return
   
case 5
   %(estado,estado)
   %(FUTURO dar error segn fixed), ahora aceptar
   return
   
case 6
   %(estado,nmero)
   %(FUTURO dar warning, ahora aceptar sin ms)
   return
   
case 7
   %(nmero,nmero)
   %dar error si son distintos
   if valor1 ~=valor2
      s1=num2str(valor1);
   	s2=num2str(valor2);
      nombreClase=ROOT.clase(ROOT.iclase).nombre;
   	ROOT.merr=['$24$' nombreClase '$' s1 '$' s2 '$'];
   	error(ROOT.merr);   
   else
      %aceptar
      return
   end
end   
   
%-------------------------------------------------------------------------------
%Entrada:
%	var1=indice en infoVars de la primera variable del alias
%	var2=indice en infoVars de la segunda variable del alias
%Salida:
%	alias=estructura alias. Donde se ha aadido los nuevos alias
%Descripcion:
%	crea un nuevo alias.
function CrearAlias(ivar1,valor1,tipo1,ivar2,valor2,tipo2)
%declarar ENTORNO
global ENTORNO

%averiguar si son compatibles
resp=SonCompatibles(ivar1,valor1,tipo1,ivar2,valor2,tipo2);

%creamos un entrada de alias con dos variables y valor ''
pos=length(ENTORNO.alias)+1;

%Si ivar1 es una variable => almacenarla
ENTORNO.alias(pos).ivars=[];
ENTORNO.alias(pos).lvars1={};
ENTORNO.alias(pos).lvars2={};

%poner la variable de sustitucion
switch resp
case 1
	ENTORNO.alias(pos).tipo=tipo1;
   if ~strcmp(tipo1,'numero')
      ENTORNO.alias(pos).ivars(1)=ivar1;
   	ENTORNO.alias(pos).lvars1{1}=ENTORNO.infoVars(ivar1).expresion;
		ENTORNO.alias(pos).lvars2{1}=ENTORNO.infoVars(ivar1).sustituida; 
   	ENTORNO.alias(pos).isust=ivar1;
   	ENTORNO.alias(pos).sust1=ENTORNO.infoVars(ivar1).expresion;
   	ENTORNO.alias(pos).sust2=ENTORNO.infoVars(ivar1).sustituida;
   	ENTORNO.alias(pos).valor=valor1;
	else
   	ENTORNO.alias(pos).isust=[];
   	ENTORNO.alias(pos).sust1='';
   	ENTORNO.alias(pos).sust2='';
   	ENTORNO.alias(pos).valor=valor1;
   end
   
   %aadir la segunda variable
   if ~strcmp(tipo2,'numero')
   	ENTORNO.alias(pos).ivars(end+1)=ivar2;
   	ENTORNO.alias(pos).lvars1{end+1}=ENTORNO.infoVars(ivar2).expresion;
		ENTORNO.alias(pos).lvars2{end+1}=ENTORNO.infoVars(ivar2).sustituida;
   end
   
case 2   
   ENTORNO.alias(pos).tipo=tipo2;
   if ~strcmp(tipo2,'numero')
      ENTORNO.alias(pos).ivars(1)=ivar2;
   	ENTORNO.alias(pos).lvars1{1}=ENTORNO.infoVars(ivar2).expresion;
		ENTORNO.alias(pos).lvars2{1}=ENTORNO.infoVars(ivar2).sustituida;
   	ENTORNO.alias(pos).isust=ivar2;
   	ENTORNO.alias(pos).sust1=ENTORNO.infoVars(ivar2).expresion;
   	ENTORNO.alias(pos).sust2=ENTORNO.infoVars(ivar2).sustituida;
   	ENTORNO.alias(pos).valor=valor2;
	else
   	ENTORNO.alias(pos).isust=[];
   	ENTORNO.alias(pos).sust1='';
   	ENTORNO.alias(pos).sust2='';
   	ENTORNO.alias(pos).valor=valor2;
   end
   
   %aadir la primera variable
   if ~strcmp(tipo1,'numero')
   	ENTORNO.alias(pos).ivars(end+1)=ivar1;
   	ENTORNO.alias(pos).lvars1{end+1}=ENTORNO.infoVars(ivar1).expresion;
		ENTORNO.alias(pos).lvars2{end+1}=ENTORNO.infoVars(ivar1).sustituida;
   end   
end

%---------------------------------------------------------------------------------
%Entrada:
%	ivar=indice de la variable a buscar
%Salida:
%	posicion=lugar dentro del alias donde se encuentra variable. [] si no se
%		encuentra
%Descripcion:
%	Busca una variable dentro de un alias
function posicion=BuscarAlias(ivar)
%declarar ENTORNO
global ENTORNO

%inicializar
posicion=[];

%Aqu variable ya es una cadena que contiene el nombre
%de una variable
for i=1:length(ENTORNO.alias)
   if any(ENTORNO.alias(i).ivars==ivar)
   	posicion=i;
   	return
   end
end

%---------------------------------------------------------------------------------
%Entrada:
%	ind=posicion donde se va a poner la nueva variable
%	ivar=indice en infoVars de la nueva variable a aadir
%Descripcion:
%	Aade a alias la variable var en la posicion ind
function AgregarAlias(ind,ivar,valor,tipo)
%declarar ENTORNO
global ENTORNO

%ver si son compatibles las variables
resp=SonCompatibles(ENTORNO.alias(ind).isust,ENTORNO.alias(ind).valor,ENTORNO.alias(ind).tipo,ivar,valor,tipo);

switch resp
case 1
   %el alias no vara, ver si se tiene que aadir la nueva variable
   if ~strcmp(tipo,'numero')
      ENTORNO.alias(ind).ivars(end+1)=ivar;
      ENTORNO.alias(ind).lvars1{end+1}=ENTORNO.infoVars(ivar).expresion;
      ENTORNO.alias(ind).lvars2{end+1}=ENTORNO.infoVars(ivar).sustituida;
   end

case 2
   %el alias varia de tipo
   ENTORNO.alias(ind).tipo=tipo;
   ENTORNO.alias(ind).isust=ivar;
   ENTORNO.alias(ind).valor=valor;
   
   %si es una variable, ponerla al principio para indicar que es la de sustitucion
   if ~strcmp(tipo,'numero')
      ENTORNO.alias(ind).ivars=[ivar ENTORNO.alias(ind).ivars];
      ENTORNO.alias(ind).lvars1={ENTORNO.infoVars(ivar).expresion ENTORNO.alias(ind).lvars1{:}};
      ENTORNO.alias(ind).lvars2={ENTORNO.infoVars(ivar).sustituida ENTORNO.alias(ind).lvars2{:}};
      ENTORNO.alias(ind).sust1=ENTORNO.infoVars(ivar).expresion;
      ENTORNO.alias(ind).sust2=ENTORNO.infoVars(ivar).sustituida;
   end
end

%--------------------------------------------------------------------------------
%Entrada:
%	alias=estructura alias
%	ind1=posicion del primer alias a unir
%	ind2=posicion del segundo alias a unir
%Salida:
%	alias=estructura alias. Alias donde se han unido las variables que ocupan los
%		lugares ind1 y ind2
%Descripion:
%	Une dos alias en uno.
function UnirAlias(ind1,ind2)
%declarar ENTORNO
global ENTORNO

%si ind1 e ind2 son iguales => no hacer nada
if ind1~=ind2

	%ind1 tiene el indice ms pequeo
	[ind1,ind2]=minmax(ind1,ind2);
   
   %averiguar si son comparibles
	ivar1=ENTORNO.alias(ind1).isust;
	valor1=ENTORNO.alias(ind1).valor;
	tipo1=ENTORNO.alias(ind1).tipo;

	ivar2=ENTORNO.alias(ind2).isust;
	valor2=ENTORNO.alias(ind2).valor;
	tipo2=ENTORNO.alias(ind2).tipo;

	resp=SonCompatibles(ivar1,valor1,tipo1,ivar2,valor2,tipo2);
   
	%copiar las variables del segundo alias en el primero
   switch resp
   case 1
      %el que manda es 1
      ENTORNO.alias(ind1).ivars=cat(2,ENTORNO.alias(ind1).ivars,ENTORNO.alias(ind2).ivars);
      ENTORNO.alias(ind1).lvars1=cat(2,ENTORNO.alias(ind1).lvars1,ENTORNO.alias(ind2).lvars1);
      ENTORNO.alias(ind1).lvars2=cat(2,ENTORNO.alias(ind1).lvars2,ENTORNO.alias(ind2).lvars2);
      %los dems campos: isust,sust1,sust2,valor no hay que modificarlos
   case 2
      %el que manda es 2
      ENTORNO.alias(ind1).ivars=cat(2,ENTORNO.alias(ind2).ivars,ENTORNO.alias(ind1).ivars);
      ENTORNO.alias(ind1).lvars1=cat(2,ENTORNO.alias(ind2).lvars1,ENTORNO.alias(ind1).lvars1);
      ENTORNO.alias(ind1).lvars2=cat(2,ENTORNO.alias(ind2).lvars2,ENTORNO.alias(ind1).lvars2);
      ENTORNO.alias(ind1).tipo=ENTORNO.alias(ind2).tipo;
      ENTORNO.alias(ind1).isust=ENTORNO.alias(ind2).isust;
      ENTORNO.alias(ind1).sust1=ENTORNO.alias(ind2).sust1;
      ENTORNO.alias(ind1).sust2=ENTORNO.alias(ind2).sust2;
      ENTORNO.alias(ind1).valor=ENTORNO.alias(ind2).valor;
   end
   
	%eliminar ind2
	ENTORNO.alias(ind2)=[];
end

%---------------------------------------------------------------------------------------------------------
function [minimo,maximo]=minmax(a,b)
if a<=b
   minimo=a;
   maximo=b;
else
   minimo=b;
   maximo=a;
end
