%Entrada:
%	entorno=estructura entorno, donde ya estn definidos los siguientes campos:
%		infoVars,infoMatrizIncidencia,ecuaciones
%	error=cadena que describe el error que se ha producido
%%Salida:
%	nEntorno= nueva estructura entorno. Aade el campo Bloques
%Descripcion:
%	Genera la estructura de bloques de un sistema de ecuaciones
%Modificado 01/08/03 Antonio Belmonte
function GenerarBloques
%declarar ROOT ENTORNO
global ROOT ENTORNO

%obtener datos
idesconocidas=ObtenerIndiceVars(ENTORNO.infoVars,{'real' 'derivada'});
iparametros=ObtenerIndiceVars(ENTORNO.infoVars,'parametro');
%iconstantes=ObtenerIndiceVars(ENTORNO.infoVars,'constante');
p=ENTORNO.infoMatrizIncidencia.permEcs;
q=ENTORNO.infoMatrizIncidencia.permVars;
r=ENTORNO.infoMatrizIncidencia.r;
s=ENTORNO.infoMatrizIncidencia.s;

%Hay tantos bloques como tamao de r-1
%nota: el tamao de r y s son iguales

%Almacenar las variables de salida
for i=1:length(r)-1
   %averiguar las salidas del bloque.
   permVars=q(s(i):s(i+1)-1);
   ENTORNO.bloques(i).salidas=idesconocidas(permVars);
      
   %Averiguar las entradas del bloque. Estas son todas las variables de las 
   %que depende el bloque. Se calculan como todas las variables que 
   %aparecen en las ecuaciones menos las variables de la salida
   %entradas=[];
   %for j=r(i):r(i+1)-1
   %   entradas=UnirVector(entradas,[ENTORNO.ecuaciones(p(j)).vars]);
   %end
   %entradas=QuitarVector(entradas,ENTORNO.bloques(i).salidas);
   %entradas=QuitarVector(entradas,iparametros);
   %entradas=QuitarVector(entradas,iconstantes);
   indices=[r(i):r(i+1)-1];
   entradas=sort([ENTORNO.ecuaciones(p(indices)).vars]);
   J=find(entradas(1:end-1)==entradas(2:end));
   entradas(J)=[];
   
   [w,P]=sort([entradas ENTORNO.bloques(i).salidas iparametros]);
   J=find(w(1:end-1)==w(2:end));
   entradas(P(J))=[];
   
   ENTORNO.bloques(i).entradas=entradas;
      
   %Averiguar las ecuaciones que forman parte del bloque
   permEcs=p(r(i):r(i+1)-1);
   [ecaux(1:r(i+1)-r(i)).expresion]=deal(ENTORNO.ecuaciones(permEcs).sustituida);
   
   %Averiguar el tipo de bloque
   if SonLineales(ecaux,ENTORNO.bloques(i).salidas)
      %Son lineales respecto las salidas. Despejar las salidas.
      %La ecuacion es de la forma variable=expresion.
      
      cad=['sol=solve(' '''' ENTORNO.ecuaciones(p(r(i))).sustituida ''''];
      for j=r(i)+1:r(i+1)-1
      	cad=[cad ',' '''' ENTORNO.ecuaciones(p(j)).sustituida ''''];
      end
     
      %generar variables respecto a las cuales vamos a despejar
      for j=1:length(ENTORNO.bloques(i).salidas)
         cad=[cad ',' '''' ENTORNO.infoVars(ENTORNO.bloques(i).salidas(j)).sustituida  ''''];
      end
      cad=[cad ');'];
      
      
      % nuevo
      syms(ENTORNO.infoVars.sustituida);
      cad=strrep(cad,'=','==');
      cad=strrep(cad,'sol=','sol');
      cad=strrep(cad,'''','');
      
      %despejar
      eval(cad);
 %     syms(ENTORNO.infoVars.sustituida)
      
      
     
      %Guardar soluciones en el bloque. Hay que distinguir cuando sol es solo un elemento de
      %cuando es una estructura
      if length(ENTORNO.bloques(i).salidas)==1
         var=ENTORNO.infoVars(ENTORNO.bloques(i).salidas(1)).sustituida;
         ENTORNO.bloques(i).sustituidas{1}=[var '=' char(sol)];
      else
         for j=1:length(ENTORNO.bloques(i).salidas)
            var=ENTORNO.infoVars(ENTORNO.bloques(i).salidas(j)).sustituida;
         	cad=sprintf('ENTORNO.bloques(%d).sustituidas{%d}=[''%s = '' char(sol.%s)];',i,j,var,var);
            eval(cad);
         end
      end
      
      %Tipo de bloque
      ENTORNO.bloques(i).tipo='explicito';
   else
   	%la ecuacin no es lineal. Transforma la ecuacion
      %a=b en a-b=0, almacenandose solo a-b 
      for j=r(i):r(i+1)-1
         ENTORNO.bloques(i).sustituidas{j-r(i)+1}=TransformarEcuacion(ENTORNO.ecuaciones(p(j)).sustituida);
      end
      
   	%tipo de bloque
      ENTORNO.bloques(i).tipo='implicito';
   end
end %for i

%------------------------------------------------------------------------------
%Entrada:
%	ec: cadena que contiene una ecuacion de la forma expr1=expr2
%Salida:
%	expr: expr1-expr2
%Descripcion:
%	Transforma una ecuacin en otra del tipo expr1-expr2=0
function expr=TransformarEcuacion(ec)
%procesar ecuacion
token('open',ec);

%leer el miembro izquierdo de la ecuacion
[tk,s]=token;
mi='';
while ~strcmp(tk,'igual')
   mi=[mi s];
   
   %siguiente token
   [tk,s]=token;
end

%despreciar el igual
%leer el md, cambindole el signo
[tk,s]=token;
md='';
parentesis=0; %nmero de parentesis abiertos
while ~strcmp(tk,'fin')
   %procesar tk
   switch tk
   case 'mas'
      if ~parentesis
         %cambiar el signo
         md=[md '-'];
      else
         %dentro de un parntesis no se cambia el signo
         md=[md '+'];
      end
   case 'menos'   
   	if ~parentesis
         %cambiar el signo
         md=[md '+'];
      else
         %dentro de un parntesis no se cambia el signo
         md=[md '-'];
      end      
   case 'pabierto'
      parentesis=parantesis+1;
      md=[md '('];
   case 'pcerrado'
      parentesis=parentesis-1;
      md=[md ')'];
   otherwise
      %copiar s
   	md=[md s];   
   end
   
   %siguiente token
   [tk,s]=token;
end
%si no hay ningn '+' o '-' al principio de md poner '-'
if md(1)~='+' | md(1)~='-'
   md=['-' md];
end

%devolver la solucion
expr=[mi md];
 
%------------------------------------------------------------------------------
%Entrada:
%	v:vector de indices
%	w:vector de indices
%Salida:
%	v:vector de indices que resulta de hacer v U w (quitando elementos repetidos)
function v=UnirVector(v,w)
%escoger el vector de mayor longitud en v
if length(w)>length(v)
   aux=v;
   v=w;
   w=v;
end

%{v el vector de mayor longitud, w el de menor longitud}
for i=1:length(w)
   if ~any(v==w(i))
      v(end+1)=w(i);
   end
end

%------------------------------------------------------------------------------
%Entrada:
%	u:vector de indices
%	v:vector de indices
%Salida:
%	u:vector de indices que resulta de hacer v - w 
function u=QuitarVector(u,v)
[w,P]=sort([u v]);
J=find(w(1:end-1)==w(2:end));
u(P(J))=[];

%----------------------------------------------------------------------------------------
%Entrada:
%	ecs=estructura ecuacion sustituida. Ecuaciones bien formadas
%	vars=vector indices a infoVars 
%Salida:
%	resp=1 si las ecuaciones son lineales respecto las variables de vvars, 0 si 
%		    no lo son
%Descripcion:
%	Averigua sin unas ecuaciones son lineales respecto a un conjunto de variables
%Modificado 21/8/07 Antonio Belmonte
function resp=SonLineales(ecs,ivars)
%averiguar si todas las ecuaciones son lineales
for i=1:length(ecs)
   if EsLineal(ecs(i).expresion,ivars)==0
      resp=0;
      return
   end
end
resp=1;

%Entrada:
%	ec=cadena que contiene una ecuacin bien formada
%	ivars=vector de celdas de cadenas. Contiene el nombre de unas variables
%Salida:
%	lin=1 si sec es lineal respecto de las variables contenidas en vvars, 0 en
%		otro caso
%Descipcion:
%	Averigua si una ecuacion es lineal respecto unas variables
%----------------------------------------------------------------------------------
function lin=EsLineal(ec,ivars)
%abrir ecuacion
token('open',ec);
[pres,lin]=m_ec(ivars);
token('close');

%Analizador sintactico. 
%Utiliza dos atributos:
%	1)presente= indica si en esa expresin aparece un variable del conjunto
%	2)lineal= indica si esa expresion es lineal
%----------------------------------------------------------------------------------
function [presente,lineal]=m_ec(ivars)
%leer el miembro izquierdo
[p1,l1]=m_sum(ivars);

%Leer el igual
[tk,s]=token;

%leer el miembro derecho
[p2,l2]=m_sum(ivars);

%escribir salida
presente= p1 | p2;
lineal= l1 & l2;

%-------------------------------
function [presente,lineal]=m_sum(ivars)
%procesar
[p1,l1]=m_p(ivars);
[tk,s]=token;

if strcmp(tk,'mas')
   [p2,l2]=m_sum(ivars);
   presente=p1 | p2;
   lineal=l1 & l2;
elseif strcmp(tk,'menos')
   [p2,l2]=m_sum(ivars);
   presente=p1 | p2;
   lineal=l1 & l2;
else
   token('unread',tk,s);
   presente=p1;
   lineal=l1;
end

%-----------------------------
function [presente,lineal]=m_p(ivars)
[p1,l1]=m_pot(ivars);
[tk,s]=token;

if strcmp(tk,'por')
   [p2,l2]=m_p(ivars);
   presente=p1 | p2;
   lineal= (l1 & ~p2) | (~p1 & l2) | (~p1 & ~p2);
elseif strcmp(tk,'dividir')
   [p2,l2]=m_p(ivars);
   presente=p1 | p2;
   lineal= (l1 & ~p2) | (~p1 & ~p2);
else
   token('unread',tk,s);
   presente=p1;
   lineal=l1;
end

%--------------------------------
function [presente,lineal]=m_pot(ivars)
[p1,l1]=m_tipo(ivars);
[tk,s]=token;

if strcmp(tk,'potencia')
   [p2,l2]=m_tipo(ivars);
   presente= p1 | p2;
   lineal= ~p1 & ~p2;
else
   token('unread',tk,s);
   presente=p1;
   lineal=l1;
end

%-------------------------------------
function [presente,lineal]=m_tipo(ivars)
%declarar ENTORNO
global ENTORNO

%procesar
[tk,s]=token;
if strcmp({'entero' 'real'},tk)
   presente=0;
   lineal=1;
elseif strcmp(tk,'name') 
   %buscar la variable en infoVars
   ind=find(strcmp({ENTORNO.infoVars.sustituida},s));
   
   if isempty(BuscarVarVector(ivars,ind))
      presente=0;
   else
      presente=1;
   end
   lineal=1;
elseif strcmp(tk,'tk_funcion')
   %leer (
   [tk,s]=LeerToken;
   
   %leer argumento de la funcion
   [presente,lin]=m_sum(ivars);
   
   %leer )
   [tk,s]=token;
   
   %Para que sea lineal no debe aparecer ninguna variable dentro de la funcion
   lineal=~presente;
   
elseif strcmp(tk,'menos')
   %leer el termino
   [presente,lineal]=m_tipo(ivars);
   
elseif strcmp(tk,'pabierto')
   %aqui nos hemos encontrado una expresion
   [presente,lineal]=m_sum(ivars);
   
   %leer )
   [tk,s]=token;
else
   token('unread',tk,s);
   presente=0;
   lineal=1;
end
