%Entrada
%	infoVars: con las constantes, parametros y time
%	ecuaciones
%Salida:
%	ENTORNO. Slo modifica los campos:
%		infoVars.tipo
%		ecuaciones(i).sustituida
%		ecuaciones(i).vars
%		ecuaciones(i).lvars1
%		ecuaciones(i).lvars2
%		when(i).cc(j).sustituida
%		when(i).cc(j).vars
%		when(i).cc(j).lvars1
%		when(i).cc(j).lvars2
%		when(i).reinit(j).mivar
%		when(i).reinit(j).misustituida
%		when(i).reinit(j).mdvar
%		when(i).reinit(j).mdlvars1
%		when(i).reinit(j).mdlvars2
%	error: siempre la cadena vaca
%Descripcion:
%	Realiza las siguientes operaciones en las ecuaciones:
%		.)Calcula: lvars1, lvars2, vars, sustituida
%		.)Comprueba que la variable est definida.
%		.)Para las derivadas, las crea y pone su tipo a derivada (valor=nan y start=0)
%		.)Sustituye las constantes por su valor en las ecuaciones y eventos
function GenerarVarsEcuaciones
%declarar ROOT
global ROOT ENTORNO

%obtener el nombre de la clase
nombreClase=ROOT.clase(ROOT.iclase).nombre;

%----------------------------------------------
%---Calcular los campos de las ecuaciones---
%----------------------------------------------

for i=1:length(ENTORNO.ecuaciones)
   %crear los campos necesarios en ENTORNO
   ENTORNO.ecuaciones(i).vars=[];
   ENTORNO.ecuaciones(i).lvars1={};
   ENTORNO.ecuaciones(i).lvars2={};
   ENTORNO.ecuaciones(i).sustituida='';
   
   %Obtener los token de la ecuacin
   token('open',ENTORNO.ecuaciones(i).expresion);
   
   %procesar la ecuacion
   [tk,s]=token;
   sustituida=''; %ecuacion sustituida
   expresion='';	%ecuacin sin sustituir
   while ~strcmp(tk,'fin')
      if strcmp(tk,'name')
         %Buscar s en infoVars
         ivar=BuscarVarInfo(ENTORNO.infoVars,s);
         if isempty(ivar)
            %no encontrada la variable            
            ROOT.merr=['$1$' nombreClase '$' s '$' ENTORNO.ecuaciones(i).expresion '$']; 
            error(ROOT.merr);
         end
         
         %Si la variable es una constante => sustituir por su valor
         if strcmp(ENTORNO.infoVars(ivar).tipo,'constante')
            valor=num2str(ENTORNO.infoVars(ivar).valor);
            sustituida=[sustituida valor];   
            expresion=[expresion valor];
         else
            %si no => aadir la variable a la ecuacion
            sustituida=[sustituida ENTORNO.infoVars(ivar).sustituida];
            expresion=[expresion s];
         end
         
         %aadir la variable a la lista de vars
         if isempty(BuscarVarVector(ENTORNO.ecuaciones(i).vars,ivar))
            ENTORNO.ecuaciones(i).vars(end+1)=ivar;
            ENTORNO.ecuaciones(i).lvars1{end+1}=s;
            ENTORNO.ecuaciones(i).lvars2{end+1}=ENTORNO.infoVars(ivar).sustituida;
         end
                  
      elseif strcmp(tk,'der')
         %leer (
         [tk,s]=token; 
         
         %leer id
         [tk,s]=token; 
         
         %comprobar si la variable s est declarada
         ivar=BuscarVarInfo(ENTORNO.infoVars,s);
      	if isempty(ivar)
            %no encontrada la variable            
            ROOT.merr=['$1$' nombreClase '$' s '$' ENTORNO.ecuaciones(i).expresion '$']; 
            error(ROOT.merr);
         end
         
         %decidir que hacer segn el tipo de la variable
         switch ENTORNO.infoVars(ivar).tipo
         case 'conocida'
            %es la derivada de time que es 1
            sustituida=[sustituida 1];
            expresion=[expresion 1];
            
         case {'constante','parametro'}
            %su derivada es cero
            sustituida=[sustituida 0];
            expresion=[expresion 0];
            
         case 'estado'
            %aadir la variable
            nombre=['der(' s ')']; %nombre de la variable derivada
         	nombreSustituida=['d001' ENTORNO.infoVars(ivar).sustituida];
            sustituida=[sustituida  nombreSustituida];
            expresion=[expresion nombre];
            
            %buscar la variable derivada en infoVars
            ivard=BuscarVarInfo(ENTORNO.infoVars,nombre);
            
            %aadirla a la lista de variables de la ecuacion
            if isempty(BuscarVarVector(ENTORNO.ecuaciones(i).vars,ivard))
               ENTORNO.ecuaciones(i).vars(end+1)=ivard;
               ENTORNO.ecuaciones(i).lvars1{end+1}=nombre;
               ENTORNO.ecuaciones(i).lvars2{end+1}=nombreSustituida;
         	end
            
            %leer )
            [tk,s]=token;
            
         case 'real'
         	%cambiar su tipo a estado
        		ENTORNO.infoVars(ivar).tipo='estado';
         
         	%buscar la variable derivada de la forma der(s)
         	nombre=['der(' s ')'];
         	nombreSustituida=['d001' ENTORNO.infoVars(ivar).sustituida];
            ivard=BuscarVarInfo(ENTORNO.infoVars,nombre);
            
           	%si no existe la variable derivada, la creamos
      		if isempty(ivard)
            	ivard=length(ENTORNO.infoVars)+1;
            	ENTORNO.infoVars(ivard).expresion=nombre;
            	ENTORNO.infoVars(ivard).tipo='derivada';
            	ENTORNO.infoVars(ivard).valor=nan;
            	ENTORNO.infoVars(ivard).start=0;
            	ENTORNO.infoVars(ivard).fixed=0;
            	ENTORNO.infoVars(ivard).sustituida=nombreSustituida;
            	ENTORNO.infoVars(ivard).orden=1;
               ENTORNO.infoVars(ivard).expresionSinDerivar=s;
         	end
            
            %aadirla a la lista de variables de la ecuacion
            if isempty(BuscarVarVector(ENTORNO.ecuaciones(i).vars,ivard))
               ENTORNO.ecuaciones(i).vars(end+1)=ivard;
               ENTORNO.ecuaciones(i).lvars1{end+1}=nombre;
               ENTORNO.ecuaciones(i).lvars2{end+1}=nombreSustituida;
         	end
            
         	%aadir a la ecuacion
         	sustituida(end+1:end+length(nombreSustituida))=nombreSustituida;
            expresion(end+1:end+length(nombre))=nombre;
            
         	%leer )
            [tk,s]=token;
         end
      else
         sustituida(end+1:end+length(s))=s;
         expresion(end+1:end+length(s))=s;
      end
      
      %siguiente token
      [tk,s]=token;
   end
   
   %almacenar la ecuacin
   ENTORNO.ecuaciones(i).expresion=expresion;
  	ENTORNO.ecuaciones(i).sustituida=sustituida;
     
  	%cerrar la lectura para esta ecuacion
 	token('close');
end

%----------------------------------------
%---Calcular los campos de los eventos---
%----------------------------------------

%leer los estados
iestados=ObtenerIndiceVars(ENTORNO.infoVars,'estado');

%Escribir los campos de ENTORNO.when
for i=1:length(ENTORNO.when)
   %transformar las condiciones
   for j=1:length(ENTORNO.when(i).cc)   
      %procesar miembro izquierdo
      [vars_i,lvars1_i,lvars2_i,sustituida_i]=LeerVariablesExpresion(ENTORNO.when(i).cc(j).mi);
      
      %procesar el miembro derecho
      [vars_d,lvars1_d,lvars2_d,sustituida_d]=LeerVariablesExpresion(ENTORNO.when(i).cc(j).md);
      
      %guardar 
      ENTORNO.when(i).cc(j).vars=cat(2,vars_i,vars_d);
      ENTORNO.when(i).cc(j).lvars1=cat(2,lvars1_i,lvars1_d);
      ENTORNO.when(i).cc(j).lvars2=cat(2,lvars2_i,lvars2_d);
      
      %escribir expresion sustituida
      switch ENTORNO.when(i).cc(j).op
      case {'mayor' 'mayorigual'}
         ENTORNO.when(i).cc(j).sustituida=[sustituida_i CambiarSignoExpresion(sustituida_d)];
      case {'menor' 'menorigual'}
         ENTORNO.when(i).cc(j).sustituida=[sustituida_d CambiarSignoExpresion(sustituida_i)];
      end
   end
   
   %tranformar reinit
   for j=1:length(ENTORNO.when(i).reinit)
      %variable de estado
      ind=BuscarVarInfo(ENTORNO.infoVars,ENTORNO.when(i).reinit(j).miexpresion);
      if isempty(ind)
         %no encontrada la variable            
         ROOT.merr=['$4$' nombreClase '$' ENTORNO.when(i).reinit(j).miexpresion '$']; 
         error(ROOT.merr);
      end
      
      %rellenar ENTORNO
      ENTORNO.when(i).reinit(j).mivar=ind;
      
      %La variable sustituida es distinta, ah que escribirla de la forma xe(%d)
      indxe=BuscarVarVector(iestados,ind);
      if isempty(indxe)
         %La variable no es un estado => error
         ROOT.merr=['$19$' nombreClase '$' ENTORNO.when(i).reinit(j).miexpresion '$']; 
         error(ROOT.merr);
      end
     	ENTORNO.when(i).reinit(j).misustituida=['xe(' num2str(indxe) ')'];
      
      %procesar el miembro derecho
      [vars_d,lvars1_d,lvars2_d,sustituida_d]=LeerVariablesExpresion(ENTORNO.when(i).reinit(j).mdexpresion);
      
      %guardar 
      ENTORNO.when(i).reinit(j).mdvars=vars_d;
      ENTORNO.when(i).reinit(j).mdlvars1=lvars1_d;
      ENTORNO.when(i).reinit(j).mdlvars2=lvars2_d;
      ENTORNO.when(i).reinit(j).mdsustituida=sustituida_d;
   end
end
      
%----------------------------------------------------------------------------------------------------------------------
%Entrada:
%	expr: cadena que contiene una expresion bien formada
%Salida:
%	vars:[indices a infoVars de las variables]
%	lvars1:{lista de variables originales}
%	lvars2:{lista de variables sustituidas}
%	sustituida: cadena que contiene la expresion sustituida
%Descripcion:
%	.)Calcula vars, lvars1, lvars2, sustituida
%	.)Sustituye constantes en la expresion
function [vars,lvars1,lvars2,sustituida]=LeerVariablesExpresion(expr)
%declarar ENTORNO
global ROOT ENTORNO

%obtener el nombre de la clase
nombreClase=ROOT.clase(ROOT.iclase).nombre;

%inicializar
vars=[];
lvars1={};
lvars2={};
sustituida='';

%recorrer la expresion
token('open',expr);
while 1
   [tk,s]=token;
   switch tk
   case 'fin'
      %salir
      break
      
   case 'name'
      %buscar el identificador en infoVars
      ind=BuscarVarInfo(ENTORNO.infoVars,s);
      if isempty(ind)
         %no encontrada la variable            
         ROOT.merr=['$4$' nombreClase '$' s '$']; 
         error(ROOT.merr);
      end
      
      %si s es una constante=>la sustituimos por su valor
      if strcmp(ENTORNO.infoVars(ind).tipo,'constante')
      	s=num2str(ENTORNO.infoVars(ind).valor);	   
      else
         %si no es constante, guardarla si no estaba antes
         if isempty(BuscarVarVector(vars,ind))
      		vars(end+1)=ind;
      		lvars1{end+1}=s;
      		lvars2{end+1}=ENTORNO.infoVars(ind).sustituida;
         end
         
      	%escribir la var sustituida
      	s=ENTORNO.infoVars(ind).sustituida;
      end   
         
   case 'der'   
      %la variable es una derivada
      
      %leer (
      token;
      
      %leer var
      [tk,s]=token;
      nombre=['der(' s ')'];
      
      %leer )
      token;
      
      %decidir segn el tipo de s
      ind=BuscarVarInfo(ENTORNO.infoVars,s);
      if isempty(ind)
         %no encontrada la variable            
         ROOT.merr=['$4$' nombreClase '$' s '$']; 
         error(ROOT.merr);
      end   
      
      switch ENTORNO.infoVars(ind).tipo
      case {'constante' 'parametro'}
         %poner la sustitucion
         s='0';
      otherwise
         %buscar la variable derivada en infoVars
         ind=BuscarVarInfo(ENTORNO.infoVars,nombre);
         if isempty(ind)
            %Si no encontrada la variable => error           
         	ROOT.merr=['$19$' nombreClase '$' s '$']; 
         	error(ROOT.merr);
         end   
         
         %guardar la variable en las listas de variables
         if isempty(BuscarVarVector(vars,ind))
      		vars(end+1)=ind;
      		lvars1{end+1}=nombre;
      		lvars2{end+1}=ENTORNO.infoVars(ind).sustituida;
         end
         
         %poner la sustitucion
         s=ENTORNO.infoVars(ind).sustituida;
      end
   end
   
   %escribir la expr sustituida
   sustituida(end+1:end+length(s))=s;   
end

%terminar
token('close');

%-------------------------------------------------------------------------------------------------------------------------
%Entrada:
%	expr: cadena que contiene una expresion
%Salida:
%	sal: cadena que contien la misma expresion de la entrada con el signo opuesto
function sal=CambiarSignoExpresion(expr)
%abrir
token('open',expr);

%Si la expresin empieza sin signo => poner -
if expr(1)~='+' | expr(1)~='-'
   sal='-';
else
   sal='';
end

%procesar
parentesis=0;
while 1
   [tk,s]=token;
   switch tk
   case 'fin'
      break
      
   case 'mas'
      if ~parentesis
         sal(end+1)='-';
      else
         sal(end+1)='+';
      end
      
   case 'menos'
      if ~parentesis
         sal(end+1)='+';
      else
         sal(end+1)='-';
      end
      
   case 'pabierto'
      parentesis=parentesis+1;
      sal(end+1)='(';
      
   case 'pcerrado'
      parentesis=parentesis-1;
      sal(end+1)=')';
      
   otherwise
		sal(end+1:end+length(s))=s;      
	end
end

%cerrar
token('close');