%Descripcion:
%	Aplica Pantelides si es necesario a un conjunto de ecuaciones.
%9/08/06 Antonio Belmonte
function Pantelides
%declarar ENTORNO
global ROOT ENTORNO

%numerar las ecuaciones para mostrarlas
numEcuaciones=length(ENTORNO.ecuaciones);
for i=1:numEcuaciones
   ENTORNO.ecuaciones(i).orden=i;
   ENTORNO.ecuaciones(i).prefijo='';
   ENTORNO.ecuaciones(i).sufijo='';
end

%coger datos que nos van a hacer falta para aplicar Pantelides
ecuacionesFinales=ENTORNO.ecuaciones;
derivadasFinales=[];
singularesFinales=[];

%obtener la informacin completa de las constantes, parametros y time
ninfoVars=CopiarInfoVars(ENTORNO.infoVars,[],{'conocida' 'constante' 'parametro' 'alias'});

%Obtener una lista con los distintos tipos de variables del sistema
%Nota:
%	1) La F significa Final
%	2) La I significa Inicial
%	3) El 1 hace referencia a las variables sin sustituir y el 2 a las sustituidas
%Ej:
%	lrealesI1= lista de variables reales Iniciales sin sustituir
[lfijas1,lfijas2]=ObtenerListaVars(ENTORNO,{'conocida' 'constante' 'parametro' 'alias'},'ambas'); 
[lestadosI1,lestadoI2]=ObtenerListaVars(ENTORNO,'estado','ambas');
[lrealesI1,lrealesI2]=ObtenerListaVars(ENTORNO,'real','ambas');
[lderivadasI1,lderivadasI2]=ObtenerListaVars(ENTORNO,'derivada','ambas');
lrealesF1=lrealesI1;
lrealesF2=lrealesI2;

%este lista contiene las conocidas, constantes, parametros, alias, estados iniciales y derivadas iniciales
lfijasI1=[lfijas1 lestadosI1 lderivadasI1];

%Valores iniciales para aplicar Pantelides
ecsPantelides=ENTORNO.ecuaciones;
infoVarsPantelides=ENTORNO.infoVars;

%aplicar Pantelides
numPantelides=Registro('recuperar','pantelides');
mostrarMatrices=Registro('recuperar','matrices');
for i=1:numPantelides
   %ordenar las ecuaciones de menor a mayor segn su nmero de orden.
   ecsPantelides=OrdenarEcuaciones(ecsPantelides);
   
   %construir matriz de incidencia
   infoMatrizPantelides=ConstruirMatrizIncidencia(infoVarsPantelides,ecsPantelides);
      
   %comprobar si se tiene que mostrar paso a paso la matriz de incidencia
   if mostrarMatrices
      %------------------------------------
      %---MOSTRAR MATRICES DE INCIDENCIA---
      %------------------------------------
      
   	%obtener los nmeros de orden de las ecuaciones
      numeroEcuaciones=[ecsPantelides.orden];
      
      %ordenar las ecuaciones
      ordenadas=OrdenarEcuaciones([ecsPantelides singularesFinales]);
      
      %abrir la figura donde mostrar la informacin de la matriz de incidencia
      if isempty(findobj('tag','IM'))
         InterfazMatriz;
      end
		      
   	%Mostar la matriz de incidencia y todas las ecuaciones
      filas=size(infoMatrizPantelides.original,1);
   	columnas=size(infoMatrizPantelides.original,2);
   	verMatrizVarsEcs(infoMatrizPantelides.original, ...
      	infoMatrizPantelides.nombreColumnasOriginal,... %[1:columnas],...
      	numeroEcuaciones,... %[1:filas],...      
      	[1 filas+1],...
      	[1 columnas+1],...
      	infoMatrizPantelides.nombreColumnasOriginal, ...
      	ordenadas);
   	set(findobj('tag','IMMensaje'),'String','ORIGINAL INCIDENCE MATRIX');
   
   	%esperar a que se pulse >> o que se cierre el cuadro de dilogo
   	uiwait(findobj('tag','IM'));
   	   
   	%mostrar la matriz de incidencia ordenada y todas las ecuaciones
  		verMatrizVarsEcs(infoMatrizPantelides.ordenada, ...
   	   infoMatrizPantelides.nombreColumnasOrdenada,... %infoMatriz.permVars,...
      	numeroEcuaciones(infoMatrizPantelides.permEcs),...      
      	infoMatrizPantelides.r,...
      	infoMatrizPantelides.s,...
      	infoMatrizPantelides.nombreColumnasOriginal, ...
      	ordenadas);
   	set(findobj('tag','IMMensaje'),'String','SORTED INCIDENCE MATRIX');
   
   	%esperar a que se pulse >> o se cierre el cuadro de dilogo
   	uiwait(findobj('tag','IM'));
   end   
      
   %comprobar si la matriz de incidencia es Singular
   [resp,singulares,noSingulares]=EsSingularMatriz(infoMatrizPantelides,ecsPantelides);
   if resp==0
      %--------------------------
      %--- SISTEMA NO SINGULAR---
      %--------------------------
      
      %si no ha hecho falta derivar => tomar la matriz de incidencia y volver
      if i==1
         ENTORNO.infoMatrizIncidencia=infoMatrizPantelides;
         
         if mostrarMatrices
	         %esperar a que se pulse >>
   	      uiwait(findobj('tag','IM'));
         
      	   %cerrar el cuadro de dilogo (si existe)
         	delete(findobj('tag','IM'));         
         end
         
         %volver
         return
      end
      
      %-----------------------------
      %---cambiar ENTORNO y salir---
      %-----------------------------
      
      %cambiar las variables de estado a incognitas si hay alguna nueva ecuacin
      if ~isempty(singularesFinales)
         %escoger por cada ecuacin derivada un variable de estado de una variable
         %que est en la ecuacin singular.
      	[lestadosF1, lderivadasF1, lrealesF1]=ObtenerVarsCambianTipo(...
      		singularesFinales,lestadosI1,lrealesF1,lrealesI1);
      end   
      
      %crear nuevo infoVars
      vars=ModificarInfoVars(ENTORNO.infoVars,lestadosF1,'estado');
      if ~isempty(vars)
         ninfoVars(end+1:end+length(vars))=vars;
      end
      
      vars=ModificarInfoVars(ENTORNO.infoVars,lderivadasF1,'derivada');
      if ~isempty(vars)
         ninfoVars(end+1:end+length(vars))=vars;
      end
            
      vars=ModificarInfoVars(ENTORNO.infoVars,lrealesF1,'real');
      if ~isempty(vars)
         ninfoVars(end+1:end+length(vars))=vars;
      end
      ENTORNO.infoVars=ninfoVars;
      
      %--------------------------------------------------------------------------
      %---Como ha cambiado infoVars. Todos los ndices hay que cambiarlos en: ---
      %---	Alias																					---
      %---	Ecuaciones																			---
      %---	Eventos																				---
      %---------------------------------------------------------------------------
      
      %lista de variables de infoVars
      lista={ENTORNO.infoVars.expresion};
      
      %cambiar indices a infoVar en los alias
      for j=1:length(ENTORNO.alias)
         ivars=zeros(1,length(ENTORNO.alias(i).ivars));
         for k=1:length(ENTORNO.alias(i).lvars1)
            ivars(k)=find(strcmp(lista,ENTORNO.alias(j).lvars1{k}));            
         end
         
         %si el tipo del alias no es nmero, la variable de sustitucin es la primera
         if ~strcmp(ENTORNO.alias(j).tipo,'numero')
            isust=ivars(1);
         else
            isust=[];
         end
            
         %cambiar el alias
         ENTORNO.alias(j).ivars=ivars;
         ENTORNO.alias(j).isust=isust;
      end
      
      %cambiar indices a infoVars en las ecuacions
      ordenadas=OrdenarEcuaciones([ecuacionesFinales derivadasFinales]);
      for i=1:length(ordenadas)
         %Buscar las variables
         ivars=[];
         for j=1:length(ordenadas(i).lvars1)
           	ivars(end+1)=find(strcmp(lista,ordenadas(i).lvars1{j}));
         end
         
         %Guardar
         ordenadas(i).vars=ivars;
      end
      ENTORNO.ecuaciones=ordenadas;
      
      %cambiar indices a infoVars en las ecuacions
		iestados=ObtenerIndiceVars(ENTORNO.infoVars,'estado');
      for i=1:length(ENTORNO.when)
         
         %cambiar indices a infoVars en las cc
         for j=1:length(ENTORNO.when(i).cc)
            %Buscar las variables
            ivars=[];
         	for k=1:length(ENTORNO.when(i).cc(j).lvars1)
           		ivars(end+1)=find(strcmp(lista,ENTORNO.when(i).cc(j).lvars1(k)));
            end
            %Guardar
            ENTORNO.when(i).cc(j).vars=ivars;
         end
         
        	%cambiar indices de reinit
         for j=1:length(ENTORNO.when(i).reinit)
            %cambiar la variable  
            ENTORNO.when(i).reinit(j).mivar=find(strcmp(lista,ENTORNO.when(i).reinit(j).miexpresion));
            
            %cambiar la sustitucin que es de la forma xe(%d)
            indxe=find(iestados==ENTORNO.when(i).reinit(j).mivar);
            ENTORNO.when(i).reinit(j).misustituida=['xe(' num2str(indxe) ')'];
            
            %miembro derecho
            ivars=[];
         	for k=1:length(ENTORNO.when(i).reinit(j).mdlvars1)
           		ivars(end+1)=find(strcmp(lista,ENTORNO.when(i).reinit(j).mdlvars1(k)));
            end
            %Guardar
            ENTORNO.when(i).reinit(j).mdvars=ivars;
         end         
      end
      
      %-------------------------------------
      %---Formar la matriz de incidencia----
      %-------------------------------------
      
      ENTORNO.infoMatrizIncidencia=ConstruirMatrizIncidencia(ENTORNO.infoVars,ENTORNO.ecuaciones);
      
      %--------------------------------------------------------------------------------
      %---- Mostrar la matriz de incidencia del  conjunto final de ecs del sistema.----
      %--------------------------------------------------------------------------------
      
      if mostrarMatrices
      	%ordenadas=OrdenarEcuaciones(nentorno.ecuaciones);
      	numeroEcuaciones=[1:length(ENTORNO.ecuaciones)];
               
         verMatrizVarsEcs(ENTORNO.infoMatrizIncidencia.ordenada, ...
      		ENTORNO.infoMatrizIncidencia.nombreColumnasOrdenada,... 
      		numeroEcuaciones(ENTORNO.infoMatrizIncidencia.permEcs),...      
      		ENTORNO.infoMatrizIncidencia.r,...
      		ENTORNO.infoMatrizIncidencia.s,...
      		ENTORNO.infoMatrizIncidencia.nombreColumnasOriginal, ...
      		ENTORNO.ecuaciones);
	   	set(findobj('tag','IMMensaje'),'String','FINAL INCIDENCE MATRIX');
   
   		%esperar a que se pulse >>
         uiwait(findobj('tag','IM'));
         
         %cerrar el cuadro de dilogo (si existe)
			delete(findobj('tag','IM'));
      end
      
      %volver
      return
      
   else
      %----------------------
      %---SISTEMA SINGULAR---
      %----------------------
   
   	%derivar las ecuaciones singulares
   	derivadas=[];
   	for j=1:length(singulares)
      	%obtener las variables de esta ecuacin
      	lvars2=singulares(j).lvars2;
      
      	%quitar las variables que son parametros y conocidas
      	lvars2=Quitar(lvars2,lfijas2);
      
      	%derivar respecto de las variables
      	sustituida=Derivar(singulares(j).sustituida,'time',lvars2);
      
      	%incrementar el nmero de ecuaciones
      	numEcuaciones=numEcuaciones+1;
      
      	%obtener la ecuacion sin derivar y las variables
      	[expresion,lvars1,lvars2]=ObtenerExpresion(infoVarsPantelides,sustituida);
      
      	%almacenar el resultado de las derivadas
      	%NOTA: vars se determina en GenerarInfoVars
      	indice=length(derivadas)+1;
      	derivadas(indice).expresion=expresion;
      	derivadas(indice).vars=[];
      	derivadas(indice).lvars1=lvars1;
      	derivadas(indice).lvars2=lvars2;
      	derivadas(indice).sustituida=sustituida;
      	derivadas(indice).orden=numEcuaciones;
      	derivadas(indice).prefijo=['D ' num2str(singulares(j).orden)];
      	derivadas(indice).sufijo='';   
   	end
   
   	%agregar a las ecuaciones Finales, las ecuaciones derivadas
   	if ~isempty(derivadas)
      	%guardar las derivadas
      	if isempty(derivadasFinales)
         	derivadasFinales=derivadas;
      	else
         	derivadasFinales(end+1:end+length(derivadas))=derivadas;
      	end
      
      	%obtener las variables reales finales
      	lvars1=unique(cat(2,derivadas.lvars1,lrealesF1));
      	lrealesF1=Quitar(lvars1,lfijasI1);
      end
   
   	%almacenar las ecuaciones singulares. Poner la palabra eliminada para mostrarla despus.
   	if ~isempty(singulares)
      	%poner sufijo de eliminada
      	[singulares.sufijo]=deal('Removed');
   	
      	%guardar las ecuaciones singulares
      	if isempty(singularesFinales)
         	singularesFinales=singulares;
      	else
         	singularesFinales(end+1:end+length(singulares))=singulares;
      	end
   	end
   
   %generar las ecuaciones y el infoVars para volver a aplicar Pantelides
   	ecsPantelides=[noSingulares derivadas];
   	[infoVarsPantelides,ecsPantelides]=GenerarInfoVars(ninfoVars,ecsPantelides);
	end 
end %for

%---------------------------------------------------------------------------
%Entrada:
%	ecuaciones=estructura ecuaciones, puede estar vaca
%Salida:
%	ecuaciones= lista de ecuaciones ordenadas
%Descripcin:
%	ordena una lista de ecuaciones segn el campo orden
function ecuaciones=OrdenarEcuaciones(ecuaciones)
if ~isempty(ecuaciones)
	[ord,perm]=sort([ecuaciones.orden]);
	ecuaciones=ecuaciones(perm);
end

%----------------------------------------------------------------------------
%Salida:
%	expresion
%	lvars1: variables sin sustituir
%	lvars2: variables sustituidas
%Descripcion:
%	a partir de la ecuacion sustituida obtiene las listas de vars y la ecuacin
%	sin sustituir
function [expresion,lvars1,lvars2]=ObtenerExpresion(infoVars,sustituida)
%inicializar
lvars1={};
lvars2={};
expresion='';

%Obtener una lista con todas las variables de infoVars sustituidas
lista={infoVars.sustituida};

%recorrer la ecuacion sustituida
token('open',sustituida);
[tk,s]=token;
while ~strcmp(tk,'fin')
   switch tk
   case 'name'
      %Si empieza por d es una derivada
      if s(1)=='d'
         %obtener el orden de la derivada
         orden=str2num(s(2:4));
         
         %obtener el nombre de la variable sin derivar
         nombre=s(5:end);
         ind=find(strcmp(lista,nombre));
         
         %escribir la variable derivada der(variable,orden)
         if orden>1
            var=['der('  infoVars(ind).expresion ',' num2str(orden) ')'];
         else   
            var=['der(' infoVars(ind).expresion ')'];
         end
         
         %guardar en las listas
         lvars1{end+1}=var;
         lvars2{end+1}=s;
      else
         %variable normal
         ind=find(strcmp(lista,s));
         var=infoVars(ind).expresion;
         
         %guardar en las listas
         lvars1{end+1}=var;
         lvars2{end+1}=s;
      end
   otherwise
      var=s;
   end
   
   %guardar
   expresion(end+1:end+length(var))=var;
   
   %siguiente token
   [tk,s]=token;
end      

%-------------------------------------------------------------------------------
%Entrada:
%	infoVars: estructura infoVars donde se han incluido las constantes, los parmetros y
%				 time
%	ecuaciones: estructura ecuaciones con el campos lvars1 y lvars2 vlido
%Salida:
%	infoVars: con todas las variables ya aadidas
%	ecuaciones: con el campo vars puesto
%Descripcion:
%	Genera infoVars a partir de las variables de las ecuaciones para aplicacacion de Pantelides
function [infoVars,ecuaciones]=GenerarInfoVars(infoVars,ecuaciones)
%Inicializar
info=[];

%Crear la informacin de todas las variables
for i=1:length(ecuaciones)
   for j=1:length(ecuaciones(i).lvars1)
      %obtener el nombre de la variable j
      var=ecuaciones(i).lvars1{j};
      varSustituida=ecuaciones(i).lvars2{j};
      
      if length(var)>4 & strcmp(var(1:4),'der(')
         %ver si es de la forma der(variable) o der(variable,orden)
         coma=find(var==',');
                        
         if ~isempty(coma)
            %orden>1
            nombre=var(5:coma-1);
            orden=str2num(var(coma+1:end-1));
         else
            %orden=1
            nombre=var(5:end-1);
            orden=1;
         end
         
         %poner el nombre de la sustituida= nombre sustituido de la variable sin derivar
         nombreSustituida=varSustituida(5:end);
      else
         %variable normal
         nombre=var;
         nombreSustituida=varSustituida;
         orden=0;
      end

		%buscarla en info la variable
      if isempty(info)
         %Como todava no ha sido creada la estructura, damos vaco
         pos=[];
      else
      	pos=find(strcmp({info.expresion},nombre));
      end
      
      %Aadirla si no est
      if isempty(pos)
         %almacenar
         pos=length(info)+1;
         info(pos).expresion=nombre;
         info(pos).sustituida=nombreSustituida;
         info(pos).orden=orden;
         info(pos).ref=[];
      else
         %Si el nuevo orden es mayor=> cambiar el orden
         if orden>info(pos).orden
            info(pos).orden=orden;
         end
      end
      
      %guardar la referencia a la ecuacion
      if length(info(pos).ref)<=orden
         info(pos).ref(orden+1).ecs(1)=i;      
		else   
         info(pos).ref(orden+1).ecs(end+1)=i;      
		end
   end
end

%crear infoVars
for i=1:length(info)
   pos=find(strcmp({infoVars.expresion},info(i).expresion));
   
   %si no est => aadir la variable de info a infoVars
   if isempty(pos)
      %generar las variables
      for j=0:info(i).orden
         %definir el tipo
         if j<info(i).orden
            tipo='estado';
         else
            if info(i).orden>0
               tipo='derivada';
            else
               tipo='real';
            end
         end
         
         %definir la expresion
         if j==0
            expresion=info(i).expresion;
            sustituida=info(i).sustituida;
         elseif j==1
            expresion=['der(' info(i).expresion ')'];
            sustituida=['d001' info(i).sustituida];
         else
            expresion=['der(' info(i).expresion ', ' num2str(j) ')'];
            prefijo=['000' num2str(j)];
            sustituida=['d' prefijo(end-2:end) info(i).sustituida];
			end
         
         %guardarlo en infoVars
         pos=length(infoVars)+1;
         infoVars(pos).expresion=expresion;
         infoVars(pos).tipo=tipo;
         infoVars(pos).valor=0;
         infoVars(pos).start=0;
         infoVars(pos).fixed=0;
         infoVars(pos).orden=0;
         infoVars(pos).sustituida=sustituida;
         infoVars(pos).expresionSinDerivar=info(i).expresion;
      end
   end
end

%Borrar la lista de variables de las ecuaciones
[ecuaciones.vars]=deal([]);

%construir vars de cada ecuacion
lista={infoVars.expresion};
for i=1:length(info)
   for j=1:length(info(i).ref)
      %construir la expresion de la variables
      %nota orden=j-1;
      if  j==1
         expresion=info(i).expresion;
      elseif j==2
         expresion=['der(' info(i).expresion ')'];
      else
         expresion=['der(' info(i).expresion ', ' num2str(j-1) ')'];
      end
      
      %buscar la expresion en lista
      ind=find(strcmp(lista,expresion));
      
      %asignar esta variable a todas las ecuaciones de referencia
      vector=info(i).ref(j).ecs;
      for k=1:length(info(i).ref(j).ecs)
         ecuaciones(vector(k)).vars(end+1)=ind;
      end
   end
end

%--------------------------------------------------------------------------------
%Entrada:
%  infoMatriz
%	ecuaciones
%Salida:
%	resp= 1 si la matriz de incidencia es singular
%			0 si la matriz de incidencia es no singular
%	ecuacionesSingulares=conjunto de ecuaciones singulares
%	ecuacionesNoSingulares=conjunto de ecuaciones que no producen la singularidad, que
%		son todas menos las sigulares.
%Descripcion:
%	Averigua si una matriz de incidencia es singular
function [resp,ecuacionesSingulares,ecuacionesNoSingulares]=EsSingularMatriz(infoMatriz,ecuaciones)
%inicializar
resp=0;
ecuacionesSingulares=[];
ecuacionesNoSingulares=ecuaciones;

%coger el nmero de filas del primer bloque
numFilas=infoMatriz.r(2)-infoMatriz.r(1)-1;
%coger el nmero de columnas del primer bloque
numColumnas=infoMatriz.s(2)-infoMatriz.s(1)-1;

%si no son iguales numFilas y numColumnas, entonces es singular
if numFilas~=numColumnas 
	%obtener las ecuaciones segn la matriz de incidencia ordenada
	ecuacionesOrdenadas=ecuaciones(infoMatriz.permEcs);

	%obtener las ecuaciones que producen la singularidad
	ecuacionesSingulares=ecuacionesOrdenadas(infoMatriz.r(1):infoMatriz.r(2)-1);

	%quitar las ecuaciones que producen la singularidad
	ecuacionesNoSingulares=ecuacionesOrdenadas(infoMatriz.r(2):length(ecuaciones));

	%devolver que el sistema es singular
   resp=1;
end

%---------------------------------------------------------------------------------
%Entrada:
%	singulares=ecuaciones singulares
%	listaEstado=lista de variables de estado iniciales
%	listaDesconocidasFinales=lista de variables desconocidas despues de aplicar Pantelides
%  listaDesconocidasIniciales=lista de variables desconocidas antes de aplicar Pantelides
%Salida:
%	nlistaEstado=nueva lista de variables de estado
%	nlistaDerivada=derivada de las variables de estado
%	nlistaDesconocidas=nueva lista de variables desconocidas
%Descripcion:
%	obtiene las variables de estado que pasan a ser desconocidas una vez que se aplica
%	Pantelides
function [nlistaEstado,nlistaDerivada,nlistaReal]=ObtenerVarsCambianTipo(singulares,...
   listaEstado,listaRealFinales,listaRealIniciales)
%obtener todas las variables de las ecuaciones derivadas
listaVarsSingulares=unique(cat(2,singulares.lvars1));

%Escogemos de las ecuaciones singulares sus variables de estado
candidatas={};
ind=[];
for i=1:length(listaEstado)
   posicion=BuscarVarLista(listaVarsSingulares,listaEstado{i});
   if ~isempty(posicion)
      ind(end+1)=i;
   end
end
if ~isempty(ind)
	[candidatas{1:length(ind)}]=deal(listaEstado{ind});
end

%calcular el nmero reales nuevas
numRealesNuevas=length(listaRealFinales)-length(listaRealIniciales);

%seleccionamos las variables que nos interesan para que dejen de ser de estado y
%pasen a se desconocidas.
%NOTA:
%	.) numSingulares= nos da el nmero de variables que se deben escoger
%	.) numCadidatas= el nmero de variables de estado candidatas
numSingulares=length(singulares)-numRealesNuevas;
numCandidatas=length(candidatas);

if numSingulares>numCandidatas
   %hay mas ecuaciones derivadas que variables de estado para escoger. Seleccionarlas todas
   seleccionadas=candidatas;
   candidatas={};
   numEscoger=numCandidatas;
else
   %hay mas candidatas que ecuaciones singulares
   seleccionadas={candidatas{1:numSingulares}};
   candidatas=Quitar(candidatas,seleccionadas);
   numEscoger=numSingulares;
end

%lanzar el asistente
[candidatas,seleccionadas]=SeleccionIncognitas('inicio',candidatas,seleccionadas,numEscoger);

%devolver las nuevas variables de estado y las nuevas desconocidas
nlistaReal=cat(2,listaRealFinales,seleccionadas);
for i=1:length(seleccionadas)
   nlistaReal{end+1}=['der(' seleccionadas{i} ')'];
end

%obtener las variables de estado
nlistaEstado=Quitar(listaEstado,seleccionadas);

%devolver las derivadas
nlistaDerivada={};
for i=1:length(nlistaEstado)
   nlistaDerivada{end+1}=['der(' nlistaEstado{i} ')'];
end

%---------------------------------------------------------------------------------------------------
%Descripcion:
%	Busca las variables de lvars en infoVarsOrigen.
%		1) Si existe en infoVarsOrigen, la copia en infoVars y le pone de tipo 'tipo'
%		2) Si no existe en infoVarsOrigen crea una nueva var de tipo tipo
function infoVars=ModificarInfoVars(infoVarsOrigen,lvars,tipo)
%inicializar
infoVars=[];

%Buscar las variables de lvars
for i=1:length(lvars)
   %Buscar la variable
   pos=BuscarVarInfo(infoVarsOrigen,lvars{i});
   if ~isempty(pos)
      %copiar la variable de infoVarsOrigen en infoVars
      if isempty(infoVars)
      	infoVars=infoVarsOrigen(pos);
      else
         infoVars(end+1)=infoVarsOrigen(pos);
      end
         
      %cambiar el tipo
      infoVars(end).tipo=tipo;
   else
      %Es una derivada que no est en infoVars
      %crear una variable de tipo 
      pos=length(infoVars)+1;
      
      %buscar la coma
      var=lvars{i};
      coma=find(var==',');
         
      %crear la variable sustituida
      if isempty(coma)
         nombre=var(5:end-1);
         orden=1;
      else
         nombre=var(5:coma-1);
         orden=str2num(var(coma+1:end-1));
      end            
      ind=BuscarVarInfo(infoVarsOrigen,nombre);
      prefijo=['000' num2str(orden)];
      sustituida=['d' prefijo(end-2:end) infoVarsOrigen(ind).sustituida];
      
      %crear la variable
      infoVars(pos).expresion=lvars{i};
      infoVars(pos).tipo=tipo;
      infoVars(pos).valor=0;
      infoVars(pos).start=0;
		infoVars(pos).fixed=0;
		infoVars(pos).sustituida=sustituida;
		infoVars(pos).orden=0;
      infoVars(pos).expresionSinDerivar=nombre;
   end
end

%---------------------------------------------------------------------------------------------------------
%Entrada:
%	lista1: lista de cadenas
%	lista2: lista de cadenas
%Descripcion:
%	lista1=lista1 U lista2
function lista1=Unir(lista1,lista2)
for i=1:length(lista2)
   ind=find(strcmp(lista1,lista2{i}));
   
   %Si no est la variable lista2{i} => aadirla
   if isempty(ind)
      lista1{end+1}=lista2{i};
   end
end

%---------------------------------------------------------------------------------------------------------
%Entrada:
%	lista1: lista de cadenas
%	lista2: lista de cadenas
%Descripcion:
%	lista1=lista1-lista2
function lista1=Quitar(lista1,lista2)
%Buscar los elementos repetidos
ind=[];
for i=1:length(lista2)
   pos=find(strcmp(lista1,lista2{i}));
   if ~isempty(pos)
      ind(end+1)=pos;
   end
end

%quitar los elementos
if ~isempty(ind)
	p=[1:length(lista1)];
	p(ind)=[];
	lista1={lista1{p}};
end
