%Entrada:
%	varargin puede ser de tres formas
%	1 forma) (cadena,'inicio')
%		cadena=cadena de la que se van a leer los tokens
%		'inicio' palabra que indica que debe inicializarse la funcion.
%	2 forma) LeerToken(token,cadena,'desleer')
%		deslee el token asociado a cadena.
%	3 forma) ningun argumento.
%		Una vez inicializada la funcion segun la forma 1, esta otra forma se utiliza
%		para obtener de uno en uno los tokens de la cadena.
%Salida:
%	s=cadena de caracteres que representa el token. Se devuelve un valor nulo cuando 
%		se inicializa la funcion o cuando se produce un error. Parametro adicional que se utiliza
%		como entero cuando se devuelve el tk_derivada
%	error=cadena que contiene una descripcion del error que se ha producido. Si no
%		se produce ninguno es la cadena vacia
%Descripcion:
%	Leer los componenentes lexicos(tokens) de una cadena
%22/07/06 Antonio Belmonte
function varargout=LeerToken(varargin)
%declarar ROOT
global ROOT

persistent fichero

%Ver el nmero de argumentos
if nargin>0
   %---leer la orden
	switch varargin{1}
	case 'open'
        %abrir fichero
        fh=fopen(varargin{2},'r');
        if fh==-1
            %leer el nmero de lnea del fichero anterior
            if ~isempty(fichero)
       %         error(['#' num2str(fichero(end).linea) '#4#' varargin{2}]);
            else
                %poner como nmero de lnea 0
        %        error(['#0#4#' varargin{2}]);
            end   
        end
   
        %guardar datos
        indice=length(fichero)+1;
        fichero(indice).nombre=varargin{2};
        fichero(indice).fh=fh;
        fichero(indice).buffer=char([fread(fh,inf)' 0]);%leer el fichero y aadir un cero como EOF
        fichero(indice).cursor=1; %posicion dentro del buffer
        fichero(indice).linea=1;  %nmero de linea
        %fichero(indice).comienzoLinea=1; %posicion del buffer donde empieza esta linea
        fichero(indice).tk=[];
        fichero(indice).s='';
        fichero(indice).lin=[];
      
    	%volver
        return
   
	case 'close'
        %cierra el ltimo fichero abierto
        %comprobar que no est cerrado ya
        try
            fclose(fichero(end).fh);      
        catch
            ROOT.merr=['#0#5#' fichero(end).nombre];
            error(ROOT.merr); 
        end
      
        %volver
        fichero(end)=[];
        return
   
    case 'closeall'
        %---cerrar todos los ficheros
        nombres=''; %nombre de los ficheros que no se han podido cerrar
        for i=1:length(fichero)
            fh=fichero(i).fh;
            try
                fclose(fh);
            catch
                nombres=[nombres ' ' fichero(i).nombre];
            end
        end
      
        %---volver
        fichero=[];
        if ~isempty(nombres)
            ROOT.merr=['#0#6#' nombres];
            error(ROOT.merr);
        end
        return
      
    case 'line'
       %---devuelve el nmero de linea del ltimo fichero
       varargout{1}=fichero(end).linea;
       return
      
	case 'name'
        %---devuelve el nombre del ltimo fichero
        if isempty(fichero)
            varargout{1}='';
        else
            varargout{1}=fichero(end).nombre;
        end
        return
      
    case 'names'
        %---devuelve el nombre de todos los ficheros abiertos
        nombres='';
        for i=1:length(fichero)
            nombres=[nombres ' ' fichero(i).nombre];
        end
        varargout{1}=nombres;
      
    case 'unread'
        %---no comprueba nada por rapidez
        fichero(end).tk=varargin{2};
        fichero(end).s=varargin{3};
        fichero(end).lin=varargin{4};
        return
      
    otherwise
        %---ignorar la orden
      return
    end
else
   %inicializar
   varargout{1}='';
   varargout{2}='';
   
   %leer un token
   %comprobar si hay algn elemento desleido
   if ~isempty(fichero(end).tk)
      varargout{1}=fichero(end).tk;
      varargout{2}=fichero(end).s;
      varargout{3}=fichero(end).lin;
      
      %borrar los tokens leidos
      fichero(end).tk=[];
      fichero(end).s=[];
      fichero(end).lin=[];
      return
   else 
      %procesar el buffer
      cursor=fichero(end).cursor;
      %buffer=fichero(end).buffer;
      
		while 1
         %leer un caracter
         inicio=cursor;
         c=fichero(end).buffer(cursor);
         varargout{3}=fichero(end).linea;
         
         if ~c
            %EOF. Volver
            varargout{1}='fin';
            varargout{2}='';
            fichero(end).cursor=cursor;
            return
            
         elseif c==char(13)
            %CR
            fichero(end).linea=fichero(end).linea+1;
            cursor=cursor+1;
      	elseif isspace(c)
         	%no hacer nada
            cursor=cursor+1;
            
         elseif c=='/'
            %puede ser comentario o dividir
            estado=1;
            while 1
               %leer caracter
               cursor=cursor+1;
               c=fichero(end).buffer(cursor);
               
            	switch estado
            	case 1
                  if c=='/'
                     %comentario de la forma //
                     estado=2;
                  elseif c=='*'
                     %comentario de la forma /*
                     estado=3;
                  else 
                     %simbolo de dividir. Volver
                     fichero(end).cursor=cursor;
            			varargout{1}='dividir';
            			varargout{2}='/';
                     return
                  end
                  
               case 2
                  %Leer un comentario de la forma //
                  %Leer hasta el final de linea
                  if ~c
      					%EOF. Volver
            			varargout{1}='fin';
            			varargout{2}='';
                     fichero(end).cursor=cursor;
            			return               
                  elseif c==char(13) | c==char(10)
                     %CR o LF,salir
                     break
                  else
                     %no hacer nada
                     estado=2;
                  end
                  
               case 3
                  %comentario de la forma /* ... */
                  %Leer hasta encontrar */
                  if ~c
                     %EOF. error
                     fichero(end).cursor=cursor;
            			error(['#' num2str(fichero(end).linea) '#7']);
                  elseif c==char(13)
                     %CR
                     fichero(end).linea=fichero(end).linea+1;
                     estado=3;
                  elseif c=='*'
                     %comprobar el fin
                     estado=4;
                  else
                     %no hacer nada
                     estado=3;
                  end
                  
               case 4
                  if ~c
                     %EOF. error
                     fichero(end).cursor=cursor;
                     error(['#' num2str(fichero(end).linea) '#7']);
                  elseif c=='/'
                     %fin del comentario. Avanzar siguiente char
                     cursor=cursor+1;
                     break
                  elseif c=='*'
                     %comprobar el fin
                     estado=4;
                  elseif c==char(13)
                     %CR
                     fichero(end).linea=fichero(end).linea+1;
                     estado=3;
                  else
                     %es otro smbolo
                     estado=3;
                  end
               end 
            end %end while
            
      	elseif (c>='0' & c<='9') | c=='.' 
         	%es un numero entero o real
            [tk,s,final]=LeerNumero(fichero(end).buffer,cursor,fichero(end).linea);
            varargout{1}=tk;
            varargout{2}=s;
            
            %volver
            fichero(end).cursor=final;
            return
            
         elseif c=='"'
            %leer una cadena
            [tk,s,final]=LeerCadena(fichero(end).buffer,cursor,fichero(end).linea);
            varargout{1}=tk;
            varargout{2}=s;
            
            %volver
            fichero(end).cursor=final;
            return
         elseif isletter(c)
            %leer palabra clave, id, name o funcion
         	estado=1;
            while 1
               %leer siguiente simbolo
               cursor=cursor+1;
               c=fichero(end).buffer(cursor);
               
            	switch estado
            	case 1
                  %leer id
                  if isletter(c) | (c>='0' & c<='9') | c=='_'
                  	estado=1;
               	elseif c=='.'
                  	estado=2;
               	else
                     %procesar la cadena leida
                     s=fichero(end).buffer(inicio:cursor-1);
                     varargout{2}=s;
                     fichero(end).cursor=cursor;
                  	break
               	end
               case 2
                  %leer name
               	if isletter(c) | c=='_'
                  	estado=3;
                  else
                     %error
                  	varargout{1}='';
                     varargout{2}='';
                     fichero(end).cursor=cursor;
                     error(['#' num2str(fichero(end).linea) '#8#' c]);
                  end
               case 3
                  if isletter(c) | (c>='0' & c<'9') | c=='_'
                     estado=3;
                  elseif c=='.'
                     estado=2;
                  else
                     %fin de name
	                  varargout{1}='name';
                     varargout{2}=fichero(end).buffer(inicio:cursor-1);
                     %volver
                     fichero(end).cursor=cursor;
                  	return
                  end
               end
         	end %fin while
         
            %procesar id
            switch(s)
         	case {'partial','class','model','package', 'record','type','connector','flow','constant','parameter','discrete',...
                     'import', 'extends' ,'equation','when','elsewhen','end','then','connect','der','reinit','and','or','not',...
                     'true','false','Real'}
            	varargout{1}=s;
               return   
         	case {'sin','cos','tan','arcsin','arccos','arctan','sinh','cosh','tanh','arcsinh','arccosh',...
               	   'arctanh','exp','log','log10','sqrt'}
            	varargout{1}='funcion';
               return
         	otherwise
            	varargout{1}='id';
            	return
         	end
         else
            %procesar smbolos
            varargout{2}=c;
                     
            switch c
            case ';'
         		varargout{1}='puntoycoma';
      		case ','
         		varargout{1}='coma';
      		case '+'
         		varargout{1}='mas';
      		case '-'
         		varargout{1}='menos';
				case '*'
		         varargout{1}='por';
		      case '^'
   		      varargout{1}='potencia';
	        	case '('
		         varargout{1}='pabierto';
		      case ')'
               varargout{1}='pcerrado';
            case '='
               %ver si es ==
               c=fichero(end).buffer(cursor+1);
               if c=='='
                  varargout{1}='comparar';
                  varargout{2}='==';
                  cursor=cursor+1;
               else
                  varargout{1}='igual';
               end
            case '>'
               %ver si es >=
               c=fichero(end).buffer(cursor+1);
               if c=='='
                  varargout{1}='mayorigual';
                  varargout{2}='==';
                  cursor=cursor+1;
               else
                  varargout{1}='mayor';
               end               
            case '<'
               %ver si es <= o <>
               c=fichero(end).buffer(cursor+1);
               if c=='='
                  varargout{1}='menorigual';
                  varargout{2}='<=';
                  cursor=cursor+1;
               elseif c=='>'
                  varargout{1}='distinto';
                  varargout{2}='<>';
                  cursor=cursor+1;
               else
                  varargout{1}='menor';
               end
            case '{'
               varargout{1}='llabierta';
            case '}'
               varargout{1}='llcerrada';
            otherwise
               %error
     			   varargout{1}='';
              	error(['#' num2str(fichero(end).linea) '#9#' c]);
            end
            
            %guardar cursor y volver
            fichero(end).cursor=cursor+1;
            return
         end
      end %fin del while
   end
end
      
%-----------------------------------------------------------------------------------------------------------------
%Entrada:
%	buffer=vector de char donde est escrito el numero
%	inicio=posicion dentro del buffer donde empieza el numero
%Salida:
%	tk='entero' si el nmero es entero
%		'real' si el nmero es real
%	s=cadena donde est almacenado el nmero
%  final= posicion del buffer donde termina el nmero+1
%Descripcion:
%	Lee un numero de un buffer. El nmero tiene la siguiente sintaxis:
%		Numero --> Entero
%					| Entero ["." [Entero]] [ ("e" | "E") ["+" | "-"] Entero ]
function [tk,s,final]=LeerNumero(buffer,inicio,linea)
final=inicio;

%procesar el buffer
estado=1;
tk='entero';
while 1
   %leer caracter
   c=buffer(final);
   
   switch estado
   case 1
      %lee un entero
      if c>='0' & c<='9'
         estado=1;
      elseif c=='.'
         %real
         tk='real';
         estado=2;
      elseif c=='e' | c=='E'
         %real
         tk='real';
         estado=4;
      else
         %final del entero. volver
         s=buffer(inicio:final-1);
         return
      end
      
   case 2
      %leer lo que hay a continuacion del punto
      if c>='0' & c<='9'
         estado=3;
      elseif c=='e' | c=='E'
         estado=4;
      else
         %final del real.volver
         s=buffer(inicio:final-1);
         return
      end
      
   case 3
      %leer el entero despues del punto
      if c>='0' & c<='9'
         estado=3;
      elseif c=='e' | c=='E'
         estado=4;
      else
         %final del real.volver
         s=buffer(inicio:final-1);
         return
      end
      
   case 4
      %Leer [ [("+" | "-")] Entero]
      if c=='+'
         estado=5;
      elseif c=='-' 
         estado=5;
      elseif c>='0' & c<='9'
         estado=6;
      else
         %error:
         error(['#' num2str(linea) '#10#' c]);
      end
      
   case 5
      %Leer obligatoriamente un dgito despues del + -
      if c>='0' & c<='9'
         estado=6;
      else
         %error
         error(['@' num2str(linea) '#11#' c]);
      end
      
   case 6
      %Leer el entero del exponente
      if c>='0' &  c<='9'
         estado=6;
      else
         %final del real. volver
         s=buffer(inicio:final-1);
         return
      end
   end%fin switch
   
   %siguiente char
   final=final+1;
end

%------------------------------------------------------------------------------------------------------------------------
%Entrada:
%	buffer: buffer donde est la cadena
%	inicio: primera posicion donde empieza la cadena
%Salida:
%	tk='cadena'
%	s=contenido de la cadena sin las comillas del inicio y del final
%	final=posicion del buffer donde termina la cadena+1
function [tk,s,final]=LeerCadena(buffer,inicio,linea)
%inicializar
s='';
final=inicio;

%leer la cadena
estado=1;
while 1
   %leer caracter
   c=buffer(final);
   
   switch estado
   case 1
      %leer las comillas iniciales
      if c=='"'
         tk='cadena';
         estado=2;
      else
         tk='';
         return
      end
      
   case 2
      %leer cualquier caracter que no sea " ni un caracter de escape
      if ~c
         %EOF. error
         error(['#' num2str(linea) '#12#' c]);
      elseif c=='"'
         %final de la cadena
         final=final+1;
         return
      elseif c=='\'
         %leer caracteres de escape
         estado=3;
      else
         %cualquier caracter
         s=[s c];
         estado=2;
      end
      
   case 3
      %interpretar caracteres de escape
      switch c
      case char(0)
         error(['#' num2str(linea) '#12#' c]);
      case ''''
         s=[s ''''];
      case '"'
         s=[s '"'];
      case '?'
         s=[s '?'];
      case '\'
         s=[s '\'];
      case 'a'
         %audible bell
         s=[s char(7)];
      case 'b'
         %backspace
         s=[s char(8)];
      case 'f'
         %line feed
         s=[s char(10)];
      case 'n'
         %CRLF
         s=[s char([13 10])];
      case 'r'
         %carrier return
         s=[s char(13)];
      case 't'
         %tabulador
         s=[s char(9)];
      case 'v'
         s=[s char(11)];
      otherwise
         %no hacer nada
         s=[s '\' c];
      end     
      estado=2; 
   end
   
   %leer el siguiente char
   final=final+1;
end
