%--------------------------------------------------------------------------------------------------------
%Entrada:
%	root: estructura root
%	iClase: indice de la clase donde se van a almacenar las ecuaciones
%Salida:
%	root: nuevo root
%	error: cadena que contiene el mensaje de error
%Descripcion:
%	La sintaxis que lee es la siguiente:
%		equation_clause --> "equation" { equation comment ";"}
%		equation --> simple_expression=expression
%					|   "connect" "(" connector_ref "," connector_ref ")" 
%					|   "when" expression "then" {equation comment ";" } "end" "when"
function equation_clause(iClase)
%declarar root como global
global ROOT

%leer equation
match('equation','@4');

%leer las ecuaciones
while 1
   [tk,s,linea]=LeerToken;   
   switch tk
   case {'mas','menos','real','entero','name','id','(','funcion','der'}
      %es una ecuacion. Hacer parser con ella. Slo comprobamos que la sintaxis sea correcta
      LeerToken('unread',tk,s,linea);
            
      %leer miembro izquioerdo de la ecuacin
      mi=ParseExpression;
      		            
      %leer el igual
      match('igual','@14');
      
      %leer el miembro derecho  
      md=ParseExpression;
    		            
      %guardar la info
      ROOT.clase(iClase).ecuacion{end+1}=[mi '=' md];
                        
   case 'connect'
      %limpiar error
      ROOT.merr='';
      
      %leer (
      match('pabierto','@12');  
             
      %leer un identificador o un name
      [tk,s,linea]=match({'id' 'name'},'@34');
      if (length(find(s=='.'))>1)
         error(['#' num2str(linea) '#3']);
      end
      id1=s;
           
      %leer coma
      match('coma','@10');
      
      %leer un identificador o un name
      [tk,s,linea]=match({'id' 'name'},'@34');
      if (length(find(s=='.'))>1)
         error(['#' num2str(linea) '#3']);
      end
      id2=s;
            
      %leer )
      match('pcerrado','@13');
            
      %comprobar que id1 <>id2
      if strcmp(id1,id2)
         error(['#' num2str(linea) '#2']);
      end
            
      %leer las clases de equivalencia del conector
      conector=ROOT.clase(iClase).conector;
            
      %1) Buscar si existe id1
      ind1=BuscarConector(conector,id1);
            
      %2) Buscar si existe id2
      ind2=BuscarConector(conector,id2);
            
      %3) Analizar cada caso
      switch 2*isempty(ind1)+isempty(ind2)
      case 0
         %ind1 apunta a un clase e ind2 apunta a otra (tal vez la misma)
         conector=UnirConectores(conector,ind1,ind2);
      case 1
         %ind1 apunta a una clase e ind2 vacio
         conector{ind1}{end+1}=id2;
      case 2
         %ind1 vacio, ind2 apunta a una clase
         conector{ind2}{end+1}=id1;
      case 3
         %ind1 vacio, ind2 vacio
         conector=CrearConector(conector,id1,id2);
      end
      
      %guardar el conector en root
      ROOT.clase(iClase).conector=conector;
             
   case 'when'
      %limpiar error
      ROOT.merr='';
           
      %leer las condiciones
      cc=expression_when;               
      
      %leer el then
      match('then','@8');
      
      %lee los reinit
      reinit=equation_when;
      
      %guarda las ecuaciones
      if ~isempty(reinit)
      	pos=length(ROOT.when)+1;
      	ROOT.when(pos).cc=cc;
         ROOT.when(pos).reinit=reinit;
     		ROOT.clase(iClase).evento(end+1)=pos;
      end        
           
      %leer {'elsewhen' expresion_when 'then' {reinit(var,expresion) ';' }}
      while 1
         %leer elsewhen
         epsilon=matchEpsilon('elsewhen','@35');
         if epsilon,break,end
         
         %leer condiciones
         cc=expression_when;
         
         %leer then
         match('then','@8');
         
         %leer las ecuaciones
         reinit=equation_when;
      
         %guardar
         if ~isempty(reinit)
            pos=length(ROOT.when)+1;
            ROOT.when(pos).cc=cc;
            ROOT.when(pos).reinit=reinit;
            ROOT.clase(iClase).evento(end+1)=pos;
         end
      end
      
      %leer end when
      match('end','@6');
      match('when','@6');
      
   otherwise
      %volver
      LeerToken('unread',tk,s,linea);
      ROOT.merr=[ROOT.merr '@20@30@7'];
      return
   end %switch
   
   %leer comentario
   comment
   
   %leer ';'
   match('puntoycoma','@9');
end %while

%-------------------------------------------------------------------------------------------------------
%Descripcion:
%Leer:
%		EXPRESSION_WHEN --> '{' relation {',' relation} '}'
%								| relation
function relacion=expression_when
%declarar ROOT
global ROOT
relacion=[];

%procesar
[tk,s,linea]=peek;
switch tk
case 'llabierta'
   %leer {
   match('llabierta','@36');
         
   %Leer '{' relation {',' relation} '}'
   while 1
      %Leer relation
      if isempty(relacion)
         relacion=relation;
      else
         relacion(end+1)=relation;
      end
      
      %leer coma
      epsilon=matchEpsilon('coma','@10');
      if epsilon,break,end
   end
   
   %leer {
   match('llcerrada','@32');
         
case {'mas' 'menos' 'entero' 'real' 'name' 'id' 'pabierto' 'funcion' 'der'}
   %relacion
   relacion=relation;
otherwise
   ROOT.err=['@' num2str(linea) ROOT.merr '@31@' s];
   error(ROOT.err);
end
  
%--------------------------------------------------------------------------------------------------------
function cad=relation
%declarar ROOT
global ROOT

%Lee arithmetic_expression op_rel arithmetic_expression
cad.mi=ParseExpression;

%leer op_rel
[tk,s]=match({'mayor' 'mayorigual' 'menor' 'menorigual'},'@26');
cad.op=tk;

%leer miembro derecho
cad.md=ParseExpression;

%--------------------------------------------------------------------------------------------------------
function reinit=equation_when
%declarar ROOT
global ROOT
reinit=[];

%leer {reinit (var,expression) comment ';'}
while 1
   %leer reinit
   epsilon=matchEpsilon('reinit','@33');
   if epsilon,break,end
            
   %leer (
   match('pabierto','@12');
         
   %leer id name
   [tk,s]=match({'id' 'name'},'@34');
   id=s;
         
   %leer ,
   match('coma','@10');
            
   %leer arithmetic_expression
   expresion=ParseExpression;
         
   %leer )
   match('pcerrado','@13');
   
   %leer comentario
   comment;
   
   %leer ;
   match('puntoycoma','@9');
   
   %almacenar la ecuacion
   pos=length(reinit)+1;
   reinit(pos).var=id;
   reinit(pos).expresion=expresion;
end

%-------------------------------------------------------------------------------------------------------------------
%Entrada:
%	conector: conjunto de clases
%	var: nombre de la variable a buscar
%Salida:
%	ind: indice de la clase encontrada o []
%Descripcion:
%	Busca una variable dentro de las clases de conector, devolviendo el
%	indice donde se encuentra
function ind=BuscarConector(conector,var)
ind=[];
for i=1:length(conector)
	if any(strcmp(conector{i},var))
   	ind=i;
      return
   end
end

%-----------------------------------------
%Entrada:
%	conector: conjunto de clases
%	var1: nombre de la variable uno
%	var2: nombre de la variable dos
%Salida:
%	conector: nuevo conector
%Descripcion:
%	crear una nueva clase formada por var1 y var2
function conector=CrearConector(conector,var1,var2)
conector{end+1}={var1 var2};

%-----------------------------------------
%Entrada:
%	conector: las clases de conector
%	ind1: indice vlido de la primera clase
%	ind2: indice vlido de la segunda clase
%Salida:
%	conector: nuevo conector
%Descripcion:
%	Une dos clases de conector en una
function conector=UnirConectores(conector,ind1,ind2)
%seleccionar el indice ms pequeo
if ind1==ind2
   %no se pueden unir, pues son las misma clase
   return
elseif ind2<ind1
   %copiar ind2 en ind1
   aux=ind1;
   ind1=ind2;
   ind2=aux;
end
%postcondicion:{ind1<ind2}

%unir los conectores en ind1
conector{ind1}={conector{ind1} conector{ind2}};

%borra ind2
conector={conector{[1:ind2-1 ind2+1:end]}}; 