package Electrical
	connector pin_p
		flow Real i;
		Real v;
	end pin_p;
	
	connector pin_n
		flow Real i;
		Real v;
	end pin_n;

	//clase parcial con dos puertos
	partial class TwoPins
		pin_p p;
		pin_n n;
		Real v,i;
	equation
		v=p.v-n.v;
		i=p.i;
		p.i+n.i=0;
	end TwoPins;

	//clase parcial con uno solo puerto
	partial class OnePin
		pin_p p;
		Real v,i;
	equation
		v=p.v;
		i=p.i;
	end OnePin;

	//componente tierra
	partial class Ground
		extends OnePin;
	equation
		v=0;
	end Ground;

	//fuente senoidal
	partial class SineVoltage
		extends TwoPins;
		parameter Real V=5 "Voltaje";
		parameter Real freqHz=50 "Frecuencia";
		parameter Real phase=0 "fase en radianes";
		constant  Real PI=3.1415926;
	equation
		v=V*sin(2*PI*freqHz*time+phase);
	end SineVoltage;

	//fuente constante
	partial class ConstantVoltage
		extends TwoPins;
		parameter Real VA=5 "Voltaje de la fuente";
	equation
		v=VA;
	end ConstantVoltage;

	//corriente senoidal
	partial class SineCurrent
		extends TwoPins;
		parameter Real IA=5 "Corriente de la fuente";
		parameter Real FREC=50 "Frecuencia de la fuente";
		constant  Real PI=3.1415926;
	equation
		i=IA*sin(2*PI*FREC*time);
	end SineCurrent;

	//corrient constante
	partial class ConstantCurrent
		extends TwoPins;
		parameter Real IA=5 "Corriente de la fuente";
	equation
		i=IA;
	end ConstantCurrent;
	

	//condensador
	partial class Capacitor
		extends TwoPins(v.start=0);
		parameter Real C=1e-6 "Capacidad del condensador";
	equation
		der(v)=i/C;
	end Capacitor;

	//bobina
	partial class Inductor
		extends TwoPins(i.start=0);
		parameter Real L=1e-3 "Valor de la bobina";
	equation
		v=L*der(i);
	end Inductor;
		

	//resistencia
	partial class Resistor
		extends TwoPins;
		parameter Real R=1e3 "Resistencia";
	equation
		v=R*i;
	end Resistor;
end Electrical;
