package electrica
	connector puerto
		flow Real i;
		Real v;
	end puerto;

	//clase parcial con dos puertos
	partial class dosPuertos
		puerto p,n;
		Real v,i;
	equation
		v=p.v-n.v;
		i=p.i;
		p.i+n.i=0;
	end dosPuertos;

	//clase parcial con uno solo puerto
	partial class unPuerto
		puerto p;
		Real v,i;
	equation
		v=p.v;
		i=p.i;
	end unPuerto;

	//componente tierra
	partial class Tierra
		extends unPuerto;
	equation
		v=0;
	end Tierra;

	//fuente senoidal
	partial class Fuente
		extends dosPuertos;
		parameter Real VA=5 "Voltaje de la fuente";
		parameter Real FREC=50 "Frecuencia de la fuente";
		constant Real PI=3.1415926;
	equation
		v=VA*sin(2*PI*FREC*time);
	end Fuente;

	//fuente constante
	partial class FuenteConstante
		extends dosPuertos;
		parameter Real VA=5 "Voltaje de la fuente";
	equation
		v=VA;
	end FuenteConstante;

	//condensador
	partial class Condensador
		extends dosPuertos(v.start=0);
		parameter Real C=1e-6 "Capacidad del condensador";
	equation
		der(v)=i/C;
	end Condensador;

	//bobina
	partial class Bobina
		extends dosPuertos(i.start=0);
		parameter Real L=1e-3 "Valor de la bobina";
	equation
		v=L*der(i);
	end Bobina;
		

	//resistencia
	partial class Resistencia
		extends dosPuertos;
		parameter Real R=1e3 "Resistencia";
	equation
		v=R*i;
	end Resistencia;
end electrica;
